"""
"""
from builtins import object
from future import standard_library
standard_library.install_aliases()

from io import BytesIO
import logging
import logging.config
import os
import shutil
import socket
import tempfile
import time
import unittest
from http.client import BadStatusLine, IncompleteRead, HTTPException

import cherrypy
from nose.plugins.attrib import attr

from WMCore.Services.Requests import Requests
from WMCore.Services.Service import Service, isfile, cache_expired
from WMQuality.TestInitCouchApp import TestInitCouchApp as TestInit


class CrappyServer(object):
    def truncated(self):
        cherrypy.response.headers['Content-Length'] = 500
        return "Hello World!"

    truncated.exposed = True


class SlowServer(object):
    def slow(self):
        time.sleep(300)
        return "Hello World!"

    slow.exposed = True


class CrappyRequest(Requests):
    def makeRequest(self, uri=None, data=None, verb='GET', incoming_headers=None,
                    encoder=True, decoder=True, contentType=None):
        # METAL \m/
        raise BadStatusLine(666)


class RegularServer(object):
    def regular(self):
        return "This is silly."

    regular.exposed = True


class BackupServer(object):
    def regular(self):
        return "This is nuts."

    regular.exposed = True


class ServiceTest(unittest.TestCase):
    def setUp(self):
        """
        Setup for unit tests
        """
        self.testInit = TestInit(__file__)
        self.testDir = self.testInit.generateWorkDir()
        testname = self.id().split('.')[-1]

        logging.basicConfig(level=logging.DEBUG,
                            format='%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
                            datefmt='%m-%d %H:%M',
                            filename='service_unittests.log',
                            filemode='w')

        logger_name = 'Service%s' % testname.replace('test', '', 1)

        self.logger = logging.getLogger(logger_name)

        # self.cache_path = tempfile.mkdtemp()
        test_dict = {'logger': self.logger,
                     'endpoint': 'https://github.com/dmwm'}

        self.myService = Service(test_dict)

        test_dict['endpoint'] = 'http://cmssw-test.cvs.cern.ch/cgi-bin/cmssw.cgi'
        self.myService2 = Service(test_dict)
        self.testUrl = 'http://cern.ch'

        self.port = 8888
        cherrypy.config.update({'server.socket_port': self.port})

    def tearDown(self):
        self.testInit.delWorkDir()
        # There was old code here to see if the test passed and send a message to
        # self.logger.info It broke in 2.7, so if needed find a supported way to do it
        return

    def testIsFile(self):
        """
        Test the `isfile` utilitarian function
        """
        f = tempfile.NamedTemporaryFile(prefix="testIsFile", delete=True)
        self.assertTrue(isfile(f))
        f.close()
        self.assertTrue(isfile(f))

        strio = BytesIO()
        self.assertTrue(isfile(strio))
        strio.close()
        self.assertTrue(isfile(strio))

        self.assertFalse(isfile("/data/srv/alan.txt"))
        self.assertFalse(isfile(1))
        self.assertFalse(isfile(None))

    def testCacheExpired(self):
        """
        Test the `cache_expired` utilitarian function. Delta is in hours
        """
        # file-like object is always considered expired
        fcache = tempfile.NamedTemporaryFile(prefix="testIsFile", delete=True)
        self.assertTrue(cache_expired(fcache, delta=0))
        self.assertTrue(cache_expired(fcache, delta=100))
        fcache.close()
        self.assertTrue(cache_expired(fcache, delta=0))
        self.assertTrue(cache_expired(fcache, delta=100))

        # path to a file that does not exist, always expired
        newfile = fcache.name + 'testCacheExpired'
        self.assertTrue(cache_expired(newfile, delta=0))
        self.assertTrue(cache_expired(newfile, delta=100))

        # now create and write something to it
        with open(newfile, 'w') as f:
            f.write("whatever")

        self.assertFalse(cache_expired(newfile, delta=1))
        time.sleep(1)
        self.assertTrue(cache_expired(newfile, delta=0))
        self.assertFalse(cache_expired(newfile, delta=1))

    def testClear(self):
        """
        Populate the cache, and then check that it's deleted
        """
        f = self.myService.refreshCache('testClear', '/WMCore/blob/master/setup.py#L11')
        self.assertTrue(os.path.exists(f.name))
        f.close()

        self.myService.clearCache('testClear')
        self.assertFalse(os.path.exists(f.name))

    def testClearAndRepopulate(self):
        """
        Populate the cache, and then check that it's deleted
        """
        f = self.myService.refreshCache('testClear', '/WMCore/blob/master/setup.py#L11')
        self.assertTrue(os.path.exists(f.name))
        f.close()

        self.myService.clearCache('testClear')
        self.assertFalse(os.path.exists(f.name))

        f = self.myService.refreshCache('testClear', '/WMCore/blob/master/setup.py#L11')
        self.assertTrue(os.path.exists(f.name))
        f.close()

    def testCachePath(self):
        cache_path = tempfile.mkdtemp()
        myConfig = {'logger': self.logger,
                'endpoint': 'http://cmssw.cvs.cern.ch/cgi-bin/cmssw.cgi',
                'cachepath': cache_path,
                'req_cache_path': '%s/requests' % cache_path
                }
        service = Service(myConfig)
        # We append hostname to the cachepath, so that we can talk to two
        # services on different hosts
        self.assertEqual(service['cachepath'],
                         '%s/cmssw.cvs.cern.ch' % myConfig['cachepath'])
        shutil.rmtree(cache_path, ignore_errors=True)

    def testCacheLifetime(self):
        """Cache deleted if created by Service - else left alone"""
        myConfig = {'logger': self.logger,
                'endpoint': 'http://cmssw.cvs.cern.ch/cgi-bin/cmssw.cgi',
                'cacheduration': 24}
        os.environ.pop('TMPDIR', None)  # Mac sets this by default
        service = Service(myConfig)
        cache_path = service['cachepath']
        self.assertTrue(os.path.isdir(cache_path))
        del service
        self.assertFalse(os.path.exists(cache_path))

        cache_path = tempfile.mkdtemp()
        myConfig['cachepath'] = cache_path
        service = Service(myConfig)
        del service
        self.assertTrue(os.path.isdir(cache_path))

    def testCachePermissions(self):
        """Raise error if pre-defined cache permission loose"""
        cache_path = tempfile.mkdtemp()
        sub_cache_path = os.path.join(cache_path, 'cmssw.cvs.cern.ch')
        os.makedirs(sub_cache_path, 0o777)
        myConfig = {'logger': self.logger,
                'endpoint': 'http://cmssw.cvs.cern.ch/cgi-bin/cmssw.cgi',
                'cacheduration': 100,
                'cachepath': cache_path}
        self.assertRaises(AssertionError, Service, myConfig)  # it has to be 0o700

    def testCacheDuration(self):
        myConfig = {'logger': self.logger,
                'endpoint': 'http://cmssw.cvs.cern.ch/cgi-bin/cmssw.cgi',
                'cacheduration': 100
                }
        service = Service(myConfig)
        self.assertEqual(service['cacheduration'], myConfig['cacheduration'])

    def testNoCacheDuration(self):
        myConfig = {'logger': self.logger,
                'endpoint': 'http://cmssw.cvs.cern.ch/cgi-bin/cmssw.cgi',
                'cacheduration': None,
                # 'cachepath' : self.cache_path,
                # 'req_cache_path': '%s/requests' % self.cache_path
                }
        service = Service(myConfig)
        self.assertEqual(service['cacheduration'], myConfig['cacheduration'])

    def testSocketTimeout(self):
        myConfig = {'logger': self.logger,
                'endpoint': 'https://github.com/dmwm',
                'cacheduration': None,
                'timeout': 10,
                }
        service = Service(myConfig)
        service.getData('%s/socketresettest' % self.testDir, '/WMCore/blob/master/setup.py#L11')
        self.assertEqual(service['timeout'], myConfig['timeout'])

    def testStaleCache(self):
        myConfig = {'logger': self.logger,
                'endpoint': 'https://github.com/dmwm',
                'usestalecache': True,
                }
        service = Service(myConfig)
        service.getData('%s/socketresettest' % self.testDir, '/WMCore/blob/master/setup.py#L11')
        self.assertEqual(service['usestalecache'], myConfig['usestalecache'])

    def testUsingStaleCache(self):
        myConfig = {'logger': self.logger,
                'endpoint': 'https://cmssdt.cern.ch/SDT/',
                'cacheduration': 0.0005,  # cache file lasts 1.8 secs
                'timeout': 10,
                'usestalecache': True,
                # 'cachepath' : self.cache_path,
                # 'req_cache_path': '%s/requests' % self.cache_path
                }
        service = Service(myConfig)
        cache = 'stalecachetest'

        # Start test from a clear cache
        service.clearCache(cache)

        cachefile = service.cacheFileName(cache)

        self.logger.info('1st call to refreshCache - should fail, there is no cache file')
        self.assertRaises(HTTPException, service.refreshCache, cache, '/lies')

        cacheddata = 'this data is mouldy'
        with open(cachefile, 'w') as f:
            f.write(cacheddata)

        self.logger.info('2nd call to refreshCache - should pass, data comes from the valid cache')
        data = service.refreshCache(cache, '/lies').read()
        self.assertEqual(cacheddata, data)

        # power nap to avoid letting the cache expire
        time.sleep(1)
        self.logger.info('3rd call to refreshCache - should pass, cache is still valid')
        data = service.refreshCache(cache, '/lies').read()
        self.assertEqual(cacheddata, data)

        # sleep a while longer so the cache dies out
        time.sleep(2)
        self.logger.info('4th call to refreshCache - should fail, cache is dead now')
        self.assertRaises(HTTPException, service.refreshCache, cache, '/lies')

        # touch/renew the file again
        cacheddata = 'foo'
        with open(cachefile, 'w') as f:
            f.write(cacheddata)

        # disable usage of stale cache, so doesn't call the endpoint if cache is valid
        service['usestalecache'] = False
        self.logger.info('5th call to refreshCache - should pass, cache is still valid')
        data = service.refreshCache(cache, '/lies').read()
        self.assertEqual(cacheddata, data)

        # consider the cache dead
        service['cacheduration'] = 0
        time.sleep(1)
        self.logger.info('6th call to refreshCache - should fail, cache is dead now')
        self.assertRaises(HTTPException, service.refreshCache, cache, '/lies')

    def testCacheFileName(self):
        """Hash url + data to get cache file name"""
        hashes = {}
        inputdata = [{}, {'fred': 'fred'},
                     {'fred': 'fred', 'carl': [1, 2]},
                     {'fred': 'fred', 'carl': ["1", "2"]},
                     {'fred': 'fred', 'carl': ["1", "2"], 'jim': {}}
                     ]
        for data in inputdata:
            thishash = self.myService.cacheFileName('bob', inputdata=data)
            thishash2 = self.myService2.cacheFileName('bob', inputdata=data)
            self.assertNotEqual(thishash, thishash2)
            self.assertTrue(thishash not in hashes, '%s is not unique' % thishash)
            self.assertTrue(thishash2 not in hashes,
                            '%s is not unique' % thishash2)
            hashes[thishash], hashes[thishash2] = None, None

    def testNoCache(self):
        """Cache disabled"""
        myConfig = {'logger': self.logger,
                'endpoint': 'https://github.com/dmwm',
                'cachepath': None,
                }
        service = Service(myConfig)

        self.assertEqual(service['cachepath'], myConfig['cachepath'])
        self.assertEqual(service['requests']['cachepath'], myConfig['cachepath'])
        self.assertEqual(service['requests']['req_cache_path'], myConfig['cachepath'])

        out = service.refreshCache('shouldntbeused', '/').read()
        self.assertTrue('html' in out)

    @attr("integration")
    def notestTruncatedResponse(self):
        """
        _TruncatedResponse_

        """
        cherrypy.tree.mount(CrappyServer())
        cherrypy.engine.start()
        FORMAT = '%(message)s'
        logging.basicConfig(format=FORMAT)
        dummyLogger = logging.getLogger('john')
        test_dict = {'logger': self.logger, 'endpoint': 'http://127.0.0.1:%i/truncated' % self.port,
                     'usestalecache': True}
        myService = Service(test_dict)
        self.assertRaises(IncompleteRead, myService.getData, 'foo', '')
        cherrypy.engine.exit()
        cherrypy.engine.stop()

    @attr("integration")
    def notestSlowResponse(self):
        """
        _SlowResponse_

        """
        cherrypy.tree.mount(SlowServer())
        cherrypy.engine.start()
        FORMAT = '%(message)s'
        logging.basicConfig(format=FORMAT)
        dummyLogger = logging.getLogger('john')
        test_dict = {'logger': self.logger, 'endpoint': 'http://127.0.0.1:%i/slow' % self.port,
                     'usestalecache': True}
        myService = Service(test_dict)
        startTime = int(time.time())
        self.assertRaises(socket.timeout, myService.getData, 'foo', '')
        self.assertTrue(int(time.time()) - startTime < 130,
                        "Error: Timeout took too long")
        cherrypy.engine.exit()
        cherrypy.engine.stop()

    def testBadStatusLine(self):
        """
        _BadStatusLine_

        """
        FORMAT = '%(message)s'
        logging.basicConfig(format=FORMAT)
        dummyLogger = logging.getLogger('john')
        test_dict = {'logger': self.logger, 'endpoint': 'http://127.0.0.1:%i/badstatus' % self.port,
                     'usestalecache': True}
        myService = Service(test_dict)
        # Have to fudge the status line in the Request object as cherrypy won't
        # Allow bad statuses to be raised
        myService['requests'] = CrappyRequest('http://bad.com', {})
        self.assertRaises(BadStatusLine, myService.getData, 'foo', '')

    @attr("integration")
    def notestZ_InterruptedConnection(self):
        """
        _InterruptedConnection_

        What happens if we shut down the server while
        the connection is still active?

        Confirm that the cache works as expected
        """

        cherrypy.tree.mount(RegularServer(), "/reg1")
        cherrypy.engine.start()
        FORMAT = '%(message)s'
        logging.basicConfig(format=FORMAT)
        dummyLogger = logging.getLogger('john')
        test_dict = {'logger': self.logger, 'endpoint': 'http://127.0.0.1:%i/reg1/regular' % self.port,
                     'usestalecache': True, "cacheduration": 0.005}
        myService = Service(test_dict)
        self.assertRaises(HTTPException, myService.getData, 'foo', 'THISISABADURL')

        data = myService.refreshCache('foo', '')
        dataString = data.read()
        self.assertEqual(dataString, "This is silly.")
        data.close()

        # Now stop the server and confirm that it is down
        cherrypy.server.stop()
        self.assertRaises(socket.error, myService.forceRefresh, 'foo', '')

        # Make sure we can still read from the cache
        data = myService.refreshCache('foo', '')
        dataString = data.read()
        self.assertEqual(dataString, "This is silly.")
        data.close()

        # Mount a backup server
        del cherrypy.tree.apps['/reg1']
        cherrypy.tree.mount(BackupServer(), "/reg1")

        # Expire cache
        time.sleep(30)
        self.assertRaises(socket.error, myService.forceRefresh, 'foo', '')

        # get expired cache results while the server is down
        data = myService.refreshCache('foo', '')
        dataString = data.read()
        self.assertEqual(dataString, "This is silly.")
        data.close()

        # Restart server
        cherrypy.server.start()

        # Confirm new server is in place
        data = myService.refreshCache('foo', '')
        dataString = data.read()
        self.assertEqual(dataString, "This is nuts.")
        data.close()

        cherrypy.engine.exit()
        cherrypy.engine.stop()

        return


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""

SingleShot EndPolicy

"""




from WMCore.WorkQueue.Policy.End.EndPolicyInterface import EndPolicyInterface

class SingleShot(EndPolicyInterface):
    """
    o No Retries
    """
    def __init__(self, **args):
        EndPolicyInterface.__init__(self, **args)

    def applyPolicy(self):
        """Apply the given policy"""
        EndPolicyInterface.applyPolicy(self)



from builtins import object, str, bytes
from future.utils import viewitems

import time
from WMCore.ReqMgr.DataStructs.Request import RequestInfo, protectedLFNs

class DataCache(object):
    # TODO: need to change to  store in  db instead of storing in the memory
    # When mulitple server run for load balancing it could have different result
    # from each server.
    _duration = 300  # 5 minitues
    _lastedActiveDataFromAgent = {}

    @staticmethod
    def getDuration():
        return DataCache._duration

    @staticmethod
    def setDuration(sec):
        DataCache._duration = sec

    @staticmethod
    def getlatestJobData():
        if (DataCache._lastedActiveDataFromAgent):
            return DataCache._lastedActiveDataFromAgent["data"]
        else:
            return {}

    @staticmethod
    def isEmpty():
        # simple check to see if the data cache is populated
        return not DataCache._lastedActiveDataFromAgent.get("data")

    @staticmethod
    def setlatestJobData(jobData):
        DataCache._lastedActiveDataFromAgent["time"] = int(time.time())
        DataCache._lastedActiveDataFromAgent["data"] = jobData

    @staticmethod
    def islatestJobDataExpired():
        if not DataCache._lastedActiveDataFromAgent:
            return True

        if (int(time.time()) - DataCache._lastedActiveDataFromAgent["time"]) > DataCache._duration:
            return True
        return False

    @staticmethod
    def filterData(filterDict, maskList):
        reqData = DataCache.getlatestJobData()

        for _, reqInfo in viewitems(reqData):
            reqData = RequestInfo(reqInfo)
            if reqData.andFilterCheck(filterDict):
                for prop in maskList:
                    result = reqData.get(prop, [])

                    if isinstance(result, list):
                        for value in result:
                            yield value
                    elif result is not None and result != "":
                        yield result

    @staticmethod
    def filterDataByRequest(filterDict, maskList=None):
        reqData = DataCache.getlatestJobData()

        if maskList is not None:
            if isinstance(maskList, (str, bytes)):
                maskList = [maskList]
            if "RequestName" not in maskList:
                maskList.append("RequestName")

        for _, reqDict in viewitems(reqData):
            reqInfo = RequestInfo(reqDict)
            if reqInfo.andFilterCheck(filterDict):

                if maskList is None:
                    yield reqDict
                else:
                    resultItem = {}
                    for prop in maskList:
                        resultItem[prop] = reqInfo.get(prop, None)
                    yield resultItem

    @staticmethod
    def getProtectedLFNs():
        reqData = DataCache.getlatestJobData()

        for _, reqInfo in viewitems(reqData):
            for dirPath in protectedLFNs(reqInfo):
                yield dirPath


"""
File       : MSPileupError.py
Author     : Valentin Kuznetsov <vkuznet AT gmail dot com>
Description: MSPileupError represents MSPileup errors
"""

# system modules
import json

# WMCore modules
from WMCore.WMException import WMException


# MSPileup error codes
MSPILEUP_GENERIC_ERROR = 1
MSPILEUP_INVALID_ERROR = 2
MSPILEUP_DUPLICATE_ERROR = 3
MSPILEUP_NOTFOUND_ERROR = 4
MSPILEUP_DATABASE_ERROR = 5
MSPILEUP_UNIQUE_ERROR = 6
MSPILEUP_SCHEMA_ERROR = 7
MSPILEUP_FRACTION_ERROR = 8


class MSPileupError(WMException):
    """
    MSPileupError represents generic MSPileup error
    """
    def __init__(self, data, msg=""):
        """
        Constructor of MSPileupError class
        """
        super().__init__(self, msg)
        self.data = data
        self.assign(msg)

    def error(self):
        """
        JSON representation of MSPileupError

        :return: MSPileupError representation
        """
        edict = {'data': self.data, 'message': self.msg, 'code': self.code, 'error': 'MSPileupError'}
        return edict

    def __str__(self):
        """
        String representation of MSPileupError

        :return: human readable MSPileupError representation
        """
        return json.dumps(self.error(), indent=4)

    def assign(self, msg="", code=0):
        """
        Assign proper message and error codes

        :param msg: string
        :param code: int
        :return: None
        """
        if msg:
            self.msg = msg
        else:
            self.msg = 'Generic MSPileup error'
        if code > 0:
            self.code = code
        else:
            self.code = MSPILEUP_GENERIC_ERROR


class MSPileupGenericError(MSPileupError):
    """
    Generic MSPileup exception
    """
    def __init__(self, data, msg=""):
        super().__init__(data, msg)
        msg = self.msg if self.msg else "generic error"
        self.assign(msg=msg, code=MSPILEUP_GENERIC_ERROR)


class MSPileupSchemaError(MSPileupError):
    """
    Schema MSPileup exception
    """
    def __init__(self, data, msg=""):
        super().__init__(data, msg)
        msg = self.msg if self.msg else "schema error"
        self.assign(msg=msg, code=MSPILEUP_SCHEMA_ERROR)


class MSPileupFractionError(MSPileupError):
    """
    Schema MSPileup exception
    """
    def __init__(self, data, msg=""):
        super().__init__(data, msg)
        msg = self.msg if self.msg else "container fraction error"
        self.assign(msg=msg, code=MSPILEUP_FRACTION_ERROR)


class MSPileupInvalidDataError(MSPileupError):
    """
    Create MSPileup exception
    """
    def __init__(self, data, msg=""):
        super().__init__(data, msg)
        msg = self.msg if self.msg else "invalid data"
        self.assign(msg=msg, code=MSPILEUP_INVALID_ERROR)


class MSPileupDuplicateDocumentError(MSPileupError):
    """
    Duplicate document MSPileup exception
    """
    def __init__(self, data, msg=""):
        super().__init__(data, msg)
        msg = self.msg if self.msg else "duplicate MSPileup document found"
        self.assign(msg=msg, code=MSPILEUP_DUPLICATE_ERROR)


class MSPileupNoKeyFoundError(MSPileupError):
    """
    No document found MSPileup exception
    """
    def __init__(self, data, msg=""):
        super().__init__(data, msg)
        msg = self.msg if self.msg else "no document found"
        self.assign(msg=msg, code=MSPILEUP_NOTFOUND_ERROR)


class MSPileupDatabaseError(MSPileupError):
    """
    Database error MSPileup exception
    """
    def __init__(self, data, msg="", err=None):
        super().__init__(data, msg)
        msg = "Dataset error"
        if err:
            msg += f" err={err}"
        self.assign(msg=msg, code=MSPILEUP_DATABASE_ERROR)


class MSPileupUniqueConstrainError(MSPileupError):
    """
    UniqueConstrain error MSPileup exception
    """
    def __init__(self, data, msg=""):
        super().__init__(data, msg)
        msg = self.msg if self.msg else "UniqueConstrainError"
        self.assign(msg=msg, code=MSPILEUP_UNIQUE_ERROR)



#!/usr/bin/env python
"""
Unittests for CPMetrics functions
"""

import unittest

from Utils.CPMetrics import flattenStats, promMetrics


class CPMetricsTests(unittest.TestCase):
    """
    unittest for CPMetrics functions
    """
    def setUp(self):
        """
        setup data to test
        """
        self.testData = {
            "Cheroot HTTPServer 4388603856": {"a": 1},
            "CherryPy Applications": {"Bytes Read/Request": 0.0}}
        return

    def tearDown(self):
        """
        Do nothing
        """
        return

    def testFlattenStats(self):
        """
        Test the flattenStats function
        """
        data = flattenStats(self.testData)
        expect = {"cherrypy_app_bytes_read_request": 0.0, "cherrypy_http_server_a": 1}
        self.assertCountEqual(data, expect)

    def testPromMetrics(self):
        """
        Test the flattenStats function
        """
        data = promMetrics(self.testData, 'test-exporter')
        self.assertEqual("# HELP" in data, True)
        self.assertEqual("# TYPE" in data, True)
        self.assertEqual("test_exporter_cherrypy_app_bytes_read_request" in data, True)
        self.assertEqual("bla-bla" in data, False)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_Workflow_t_

Unit tests for the WMBS Workflow class.
"""

from builtins import range

import threading
import unittest

from WMCore.DAOFactory import DAOFactory
from WMCore.WMBS.Fileset import Fileset
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.Workflow import Workflow
from WMQuality.TestInit import TestInit


class WorkflowTest(unittest.TestCase):
    def setUp(self):
        """
        _setUp_

        Setup the database and logging connection.  Try to create all of the
        WMBS tables.
        """
        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules=["WMCore.WMBS"],
                                useDefault=False)
        return

    def tearDown(self):
        """
        _tearDown_

        Drop all the WMBS tables.
        """
        self.testInit.clearDatabase()
        return

    def testCreateDeleteExists(self):
        """
        _testCreateDeleteExists_

        Test the create(), delete() and exists() methods of the workflow class
        by creating and deleting a workflow.  The exists() method will be
        called before and after creation and after deletion.
        """
        testWorkflow = Workflow(spec="spec.xml", owner="Simon",
                                name="wf001", task='Test', wfType="ReReco")

        self.assertEqual(testWorkflow.exists(), False,
                         "ERROR: Workflow exists before it was created")

        testWorkflow.create()

        self.assertTrue(testWorkflow.exists() > 0,
                        "ERROR: Workflow does not exist after it has been created")

        testWorkflow.create()
        testWorkflow.delete()

        self.assertEqual(testWorkflow.exists(), False,
                         "ERROR: Workflow exists after it has been deleted")
        return

    def testCreateTransaction(self):
        """
        _testCreateTransaction_

        Create a workflow and commit it to the database and then roll the
        transaction back.  Use the workflow's exists() method to verify that the
        workflow does not exist before create() is called, exists after create()
        is called and does not exist after the transaction is rolled back.
        """
        myThread = threading.currentThread()
        myThread.transaction.begin()

        testWorkflow = Workflow(spec="spec.xml", owner="Simon",
                                name="wf001", task='Test')

        self.assertEqual(testWorkflow.exists(), False,
                         "ERROR: Workflow exists before it was created")

        testWorkflow.create()

        self.assertTrue(testWorkflow.exists() > 0,
                        "ERROR: Workflow does not exist after it has been created")

        myThread.transaction.rollback()

        self.assertEqual(testWorkflow.exists(), False,
                         "ERROR: Workflow exists after the transaction was rolled back.")
        return

    def testDeleteTransaction(self):
        """
        _testDeleteTransaction_

        Create a workflow and commit it to the database.  Begin a transaction
        and delete the workflow, then rollback the transaction.  Use the
        workflow's exists() method to verify that the workflow doesn't exist
        in the database before create() is called, it does exist after create()
        is called, it doesn't exist after delete() is called and it does exist
        after the transaction is rolled back.
        """
        testWorkflow = Workflow(spec="spec.xml", owner="Simon",
                                name="wf001", task='Test')

        self.assertEqual(testWorkflow.exists(), False,
                         "ERROR: Workflow exists before it was created")

        testWorkflow.create()

        self.assertTrue(testWorkflow.exists() > 0,
                        "ERROR: Workflow does not exist after it has been created")

        myThread = threading.currentThread()
        myThread.transaction.begin()

        testWorkflow.delete()

        self.assertEqual(testWorkflow.exists(), False,
                         "ERROR: Workflow exists after it has been deleted")

        myThread.transaction.rollback()

        self.assertTrue(testWorkflow.exists() > 0,
                        "ERROR: Workflow does not exist transaction was rolled back")

        return

    def testLoad(self):
        """
        _testLoad_

        Create a workflow and then try to load it from the database using the
        following load methods:
          Workflow.LoadFromNameAndTask
          Workflow.LoadFromID
          Workflow.LoadFromSpecOwner
        """
        testWorkflowA = Workflow(spec="spec.xml", owner="Simon",
                                 name="wf001", task='Test', wfType="ReReco",
                                 priority=1000)
        testWorkflowA.create()

        testWorkflowB = Workflow(name="wf001", task='Test')
        testWorkflowB.load()

        self.assertTrue((testWorkflowA.id == testWorkflowB.id) and
                        (testWorkflowA.name == testWorkflowB.name) and
                        (testWorkflowA.spec == testWorkflowB.spec) and
                        (testWorkflowA.task == testWorkflowB.task) and
                        (testWorkflowA.wfType == testWorkflowB.wfType) and
                        (testWorkflowA.owner == testWorkflowB.owner) and
                        (testWorkflowA.priority == testWorkflowB.priority),
                        "ERROR: Workflow.LoadFromNameAndTask Failed")

        testWorkflowC = Workflow(id=testWorkflowA.id)
        testWorkflowC.load()

        self.assertTrue((testWorkflowA.id == testWorkflowC.id) and
                        (testWorkflowA.name == testWorkflowC.name) and
                        (testWorkflowA.spec == testWorkflowC.spec) and
                        (testWorkflowA.task == testWorkflowC.task) and
                        (testWorkflowA.wfType == testWorkflowC.wfType) and
                        (testWorkflowA.owner == testWorkflowC.owner) and
                        (testWorkflowA.priority == testWorkflowB.priority),
                        "ERROR: Workflow.LoadFromID Failed")

        testWorkflowD = Workflow(spec="spec.xml", owner="Simon", task='Test')
        testWorkflowD.load()

        self.assertTrue((testWorkflowA.id == testWorkflowD.id) and
                        (testWorkflowA.name == testWorkflowD.name) and
                        (testWorkflowA.spec == testWorkflowD.spec) and
                        (testWorkflowA.task == testWorkflowD.task) and
                        (testWorkflowA.wfType == testWorkflowD.wfType) and
                        (testWorkflowA.owner == testWorkflowD.owner) and
                        (testWorkflowA.priority == testWorkflowB.priority),
                        "ERROR: Workflow.LoadFromSpecOwner Failed")
        testWorkflowA.delete()
        return

    def testCreateLoadWithUserAtt(self):
        """
        _testLoad_

        Create a workflow and then try to load it from the database using the
        following load methods:
          Workflow.LoadFromNameAndTask
          Workflow.LoadFromID
          Workflow.LoadFromSpecOwner
        Test if the Workflow is created correctly.
        """
        testWorkflowA = Workflow(spec="spec.xml", owner="Simon",
                                 owner_vogroup='integration', owner_vorole='priority',
                                 name="wf001", task='Test', wfType="ReReco")
        testWorkflowA.create()

        testWorkflowB = Workflow(name="wf001", task='Test')
        testWorkflowB.load()

        self.assertTrue((testWorkflowA.id == testWorkflowB.id) and
                        (testWorkflowA.name == testWorkflowB.name) and
                        (testWorkflowA.spec == testWorkflowB.spec) and
                        (testWorkflowA.task == testWorkflowB.task) and
                        (testWorkflowA.wfType == testWorkflowB.wfType) and
                        (testWorkflowA.owner == testWorkflowB.owner),
                        "ERROR: Workflow.LoadFromNameAndTask Failed")

        testWorkflowC = Workflow(id=testWorkflowA.id)
        testWorkflowC.load()

        self.assertTrue((testWorkflowA.id == testWorkflowC.id) and
                        (testWorkflowA.name == testWorkflowC.name) and
                        (testWorkflowA.spec == testWorkflowC.spec) and
                        (testWorkflowA.task == testWorkflowC.task) and
                        (testWorkflowA.wfType == testWorkflowC.wfType) and
                        (testWorkflowA.owner == testWorkflowC.owner),
                        "ERROR: Workflow.LoadFromID Failed")

        testWorkflowD = Workflow(spec="spec.xml", owner="Simon", task='Test')
        testWorkflowD.load()

        self.assertTrue((testWorkflowA.id == testWorkflowD.id) and
                        (testWorkflowA.name == testWorkflowD.name) and
                        (testWorkflowA.spec == testWorkflowD.spec) and
                        (testWorkflowA.task == testWorkflowD.task) and
                        (testWorkflowA.wfType == testWorkflowD.wfType) and
                        (testWorkflowA.owner == testWorkflowD.owner),
                        "ERROR: Workflow.LoadFromSpecOwner Failed")

        testWorkflowE = Workflow(spec="spec_1.xml", owner="Simon",
                                 owner_vogroup='t1access', owner_vorole='t1access',
                                 name="wf002", task='Test_1', wfType="ReReco")
        testWorkflowE.create()

        self.assertTrue((testWorkflowE.id != testWorkflowA.id) and
                        (testWorkflowE.name != testWorkflowA.name) and
                        (testWorkflowE.spec != testWorkflowA.spec) and
                        (testWorkflowE.vogroup != testWorkflowA.vogroup) and
                        (testWorkflowE.vorole != testWorkflowA.vorole) and
                        (testWorkflowE.task != testWorkflowA.task) and
                        (testWorkflowE.wfType == testWorkflowA.wfType) and
                        (testWorkflowE.owner == testWorkflowA.owner),
                        "ERROR: Workflow.LoadFromSpecOwner Failed")

        testWorkflowA.delete()
        return

    def testOutput(self):
        """
        _testOutput_

        Creat a workflow and add some outputs to it.  Verify that these are
        stored to and loaded from the database correctly.
        """
        testFilesetA = Fileset(name="testFilesetA")
        testMergedFilesetA = Fileset(name="testMergedFilesetA")
        testFilesetB = Fileset(name="testFilesetB")
        testMergedFilesetB = Fileset(name="testMergedFilesetB")
        testFilesetC = Fileset(name="testFilesetC")
        testMergedFilesetC = Fileset(name="testMergedFilesetC")
        testFilesetA.create()
        testFilesetB.create()
        testFilesetC.create()
        testMergedFilesetA.create()
        testMergedFilesetB.create()
        testMergedFilesetC.create()

        testWorkflowA = Workflow(spec="spec.xml", owner="Simon",
                                 name="wf001", task='Test')
        testWorkflowA.create()

        testWorkflowB = Workflow(name="wf001", task='Test')
        testWorkflowB.load()

        self.assertEqual(len(testWorkflowB.outputMap), 0,
                         "ERROR: Output map exists before output is assigned")

        testWorkflowA.addOutput("outModOne", testFilesetA, testMergedFilesetA)
        testWorkflowA.addOutput("outModOne", testFilesetC, testMergedFilesetC)
        testWorkflowA.addOutput("outModTwo", testFilesetB, testMergedFilesetB)

        testWorkflowC = Workflow(name="wf001", task='Test')
        testWorkflowC.load()

        self.assertEqual(len(testWorkflowC.outputMap), 2,
                         "ERROR: Incorrect number of outputs in output map")
        self.assertTrue("outModOne" in testWorkflowC.outputMap.keys(),
                        "ERROR: Output modules missing from workflow output map")
        self.assertTrue("outModTwo" in testWorkflowC.outputMap.keys(),
                        "ERROR: Output modules missing from workflow output map")

        for outputMap in testWorkflowC.outputMap["outModOne"]:
            if outputMap["output_fileset"].id == testFilesetA.id:
                self.assertEqual(outputMap["merged_output_fileset"].id,
                                 testMergedFilesetA.id,
                                 "Error: Output map incorrectly maps filesets.")
            else:
                self.assertEqual(outputMap["merged_output_fileset"].id,
                                 testMergedFilesetC.id,
                                 "Error: Output map incorrectly maps filesets.")
                self.assertEqual(outputMap["output_fileset"].id,
                                 testFilesetC.id,
                                 "Error: Output map incorrectly maps filesets.")

        self.assertEqual(testWorkflowC.outputMap["outModTwo"][0]["merged_output_fileset"].id,
                         testMergedFilesetB.id,
                         "Error: Output map incorrectly maps filesets.")
        self.assertEqual(testWorkflowC.outputMap["outModTwo"][0]["output_fileset"].id,
                         testFilesetB.id,
                         "Error: Output map incorrectly maps filesets.")
        return

    def testLoadFromTask(self):
        """
        _testLoadFromTask_

        Verify that Workflow.LoadFromTask DAO correct loads the workflow by
        task.
        """
        testWorkflow1 = Workflow(spec="spec1.xml", owner="Hassen",
                                 name="wf001", task="sometask")
        testWorkflow1.create()

        myThread = threading.currentThread()
        daoFactory = DAOFactory(package="WMCore.WMBS", logger=myThread.logger,
                                dbinterface=myThread.dbi)
        loadFromTaskDAO = daoFactory(classname="Workflow.LoadFromTask")

        listFromTask = loadFromTaskDAO.execute(task=testWorkflow1.task)

        self.assertEqual(len(listFromTask), 1,
                         "ERROR: listFromTask should be 1.")
        self.assertEqual(listFromTask[0]["task"], "sometask",
                         "ERROR: task should be sometask.")
        return

    def testWorkflowOwner(self):
        """
        _testWorkflowOwner_

        Verify that the user is being added and handled correctly
        """

        owner = "Spiga"
        testWorkflow1 = Workflow(spec="spec1.xml", owner=owner,
                                 name="wf001", task="MultiUser-support")
        testWorkflow1.create()

        myThread = threading.currentThread()
        daoFactory = DAOFactory(package="WMCore.WMBS", logger=myThread.logger,
                                dbinterface=myThread.dbi)
        loadFromOwnerDAO = daoFactory(classname="Workflow.LoadFromSpecOwner")

        listFromOwner1 = loadFromOwnerDAO.execute(task=testWorkflow1.task,
                                                  dn=testWorkflow1.owner,
                                                  spec=testWorkflow1.spec)

        testWorkflow2 = Workflow(spec="spec2.xml", owner=owner,
                                 name="wf002", task="MultiUser-support")
        testWorkflow2.create()

        listFromOwner2 = loadFromOwnerDAO.execute(task=testWorkflow2.task,
                                                  dn=testWorkflow2.owner,
                                                  spec=testWorkflow2.spec)

        testWorkflow3 = Workflow(spec="spec3.xml", owner="Ciccio",
                                 name="wf003", task="MultiUser-support")
        testWorkflow3.create()

        listFromOwner3 = loadFromOwnerDAO.execute(task=testWorkflow3.task,
                                                  dn=testWorkflow3.owner,
                                                  spec=testWorkflow3.spec)

        self.assertEqual(testWorkflow1.owner, owner)
        self.assertEqual(listFromOwner1["owner"], owner)
        self.assertEqual(listFromOwner2["owner"], owner)
        self.assertEqual(listFromOwner1["owner"], listFromOwner2["owner"])
        self.assertNotEqual(listFromOwner1["owner"], listFromOwner3["owner"])
        self.assertNotEqual(listFromOwner2["owner"], listFromOwner3["owner"])
        return

    def testCountWorkflow(self):
        """
        _testCountWorkflow_

        """
        spec = "spec.py"
        owner = "moron"
        myThread = threading.currentThread()
        daoFactory = DAOFactory(package="WMCore.WMBS", logger=myThread.logger,
                                dbinterface=myThread.dbi)
        countSpecDAO = daoFactory(classname="Workflow.CountWorkflowBySpec")

        workflows = []
        for i in range(0, 10):
            testWorkflow = Workflow(spec=spec, owner=owner,
                                    name="wf00%i" % i, task="task%i" % i)
            testWorkflow.create()
            workflows.append(testWorkflow)

        self.assertEqual(countSpecDAO.execute(spec=spec), 10)

        for i in range(0, 10):
            wf = workflows.pop()
            wf.delete()
            self.assertEqual(countSpecDAO.execute(spec=spec), 10 - (i + 1))

        return

    def testWorkflowInjectMarking(self):
        """
        _testWorkflowInjectMarking_

        Test whether or not we can mark a workflow as injected or not.
        """
        owner = "moron"

        workflows = []
        for i in range(0, 10):
            testWorkflow = Workflow(spec="sp00%i" % i, owner=owner,
                                    name="wf00%i" % i, task="task%i" % i)
            testWorkflow.create()
            workflows.append(testWorkflow)

        myThread = threading.currentThread()
        daoFactory = DAOFactory(package="WMCore.WMBS", logger=myThread.logger,
                                dbinterface=myThread.dbi)
        getAction = daoFactory(classname="Workflow.GetInjectedWorkflows")
        markAction = daoFactory(classname="Workflow.MarkInjectedWorkflows")

        result = getAction.execute(injected=True)
        self.assertEqual(len(result), 0)
        result = getAction.execute(injected=False)
        self.assertEqual(len(result), 10)

        names = ['wf002', 'wf004', 'wf006', 'wf008']
        markAction.execute(names=names, injected=True)

        result = getAction.execute(injected=True)
        self.assertEqual(result, names)
        result = getAction.execute(injected=False)
        self.assertEqual(len(result), 6)
        return

    def testGetFinishedWorkflows(self):
        """
        _testGetFinishedWorkflows_

        Test that we get only those workflows which are finished, that is, workflows where
        all its subscriptions are finished and all other workflows with the same
        spec are finished too

        """

        owner = "no-one"

        # Create a bunch of worklows with "different" specs and tasks
        workflows = []
        for i in range(0, 100):
            scaledIndex = i % 10
            testWorkflow = Workflow(spec="sp00%i" % scaledIndex,
                                    owner=owner,
                                    name="wf00%i" % scaledIndex,
                                    task="task%i" % i)
            testWorkflow.create()
            workflows.append(testWorkflow)

        # Everyone will use this fileset
        testFileset = Fileset(name="TestFileset")
        testFileset.create()

        # Create subscriptions!
        subscriptions = []
        for workflow in workflows:
            subscription = Subscription(fileset=testFileset,
                                        workflow=workflow)
            subscription.create()
            subscriptions.append(subscription)

        # Check that all workflows are NOT finished
        myThread = threading.currentThread()
        daoFactory = DAOFactory(package="WMCore.WMBS", logger=myThread.logger,
                                dbinterface=myThread.dbi)

        getFinishedDAO = daoFactory(classname="Workflow.GetFinishedWorkflows")
        result = getFinishedDAO.execute()
        self.assertEqual(len(result), 0, "A workflow is incorrectly flagged as finished: %s" % str(result))

        # Mark the first 50 subscriptions as finished
        for idx, sub in enumerate(subscriptions):
            if idx > 49:
                break
            sub.markFinished()

        # No workflow is finished, none of them has all the subscriptions completed
        result = getFinishedDAO.execute()
        self.assertEqual(len(result), 0, "A workflow is incorrectly flagged as finished: %s" % str(result))

        # Now finish all workflows in wf{000-5}
        for idx, sub in enumerate(subscriptions):
            if idx < 50 or idx % 10 > 5:
                continue
            sub.markFinished()

        # Check the workflows
        result = getFinishedDAO.execute()
        self.assertEqual(len(result), 6, "A workflow is incorrectly flagged as finished: %s" % str(result))

        # Check the overall structure of the workflows
        for wf in result:
            # Sanity checks on the results
            # These are very specific checks and depends heavily on the names of task, spec and workflow
            self.assertEqual(wf[2:], result[wf]['spec'][2:],
                             "A workflow has the wrong spec-name combination: %s" % str(wf))
            self.assertTrue(int(wf[2:]) < 6,
                            "A workflow is incorrectly flagged as finished: %s" % str(wf))
            self.assertEqual(len(result[wf]['workflows']), 10,
                             "A workflow has more tasks than it should: %s" % str(result[wf]))
            for task in result[wf]['workflows']:
                self.assertEqual(len(result[wf]['workflows'][task]), 1,
                                 "A workflow has more subscriptions than it should: %s" % str(result[wf]))

        return


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
"""
WMBase.py

Created by Dave Evans on 2011-05-20.
Copyright (c) 2011 Fermilab. All rights reserved.
"""
import os
import os.path
import unittest

from nose.plugins.attrib import attr

from WMCore.WMBase import getWMBASE, getTestBase

class WMBaseTest(unittest.TestCase):


    def testA(self):

        try:
            getWMBASE()
        except Exception as ex:
            self.fail("Failed to call getWMBASE")

        return

    @attr("integration")
    def testB_TestBase(self):
        """
        _TestBase_

        See if we can use the defaults or an environment variable
        to build a testBase

        NOTE: This has to run in WMCore/test/python/WMCore_t, and
        so is listed as integration.
        """

        test = os.path.normpath(os.path.join(os.getcwd(), '../../../test/python'))
        base = getTestBase()
        self.assertEqual(base, test)
        base = getTestBase(importFlag = False)
        self.assertEqual(base, test)
        os.environ['WMCORE_TEST_ROOT'] = '/thisdirectoryshouldneverexist'
        base = getTestBase(importFlag = False)
        self.assertEqual(base, test)
        os.environ['WMCORE_TEST_ROOT'] = '/tmp'
        base = getTestBase(importFlag = False)
        self.assertEqual(base, '/tmp')

        return


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_Step.Executor.LogArchive_

Implementation of an Executor for a LogArchive step
"""
from __future__ import division

import logging
import os
import os.path
import random
import re
import signal
import tarfile
import time

import WMCore.Storage.FileManager
import WMCore.Storage.StageOutMgr as StageOutMgr
from Utils.FileTools import calculateChecksums
from WMCore.Algorithms.Alarm import Alarm, alarmHandler
from WMCore.WMException import WMException
from WMCore.WMSpec.Steps.Executor import Executor
from WMCore.WMSpec.Steps.WMExecutionFailure import WMExecutionFailure

lfnGroup = lambda j: str(j.get("counter", 0) // 1000).zfill(4)


class LogArchive(Executor):
    """
    _LogArchive_

    Execute a LogArchive Step

    """

    def pre(self, emulator=None):
        """
        _pre_

        Pre execution checks
        """

        # Are we using an emulator?
        if emulator is not None:
            return emulator.emulatePre(self.step)

        logging.info("Steps.Executors.%s.pre called", self.__class__.__name__)
        return None

    def sendLogToEOS(self, overrides, tarBallLocation, useNewStageOutCode):
        """
        :param overrides: dictionary for setting the eos lfn.
        :return: None (copy log archive to eos location.)
        Don'f fail the job if copy fails
        """
        eosStageOutParams = {}
        eosStageOutParams['command'] = "xrdcp"
        eosStageOutParams['option'] = "--wma-disablewriterecovery"
        eosStageOutParams['phedex-node'] = overrides.get('eos-phedex-node', "T2_CH_CERN")
        eosStageOutParams['lfn-prefix'] = overrides.get('eos-lfn-prefix',
                                                        "root://eoscms.cern.ch//eos/cms/store/logs/prod/recent")

        if not eosStageOutParams['lfn-prefix']:
            # if overrides for eos-lfn-prefix is set to None or "", don't copy the log to eos
            logging.info("No 'lfn-prefix' found, not writing logs to CERN EOS recent area.")
            return
        elif not self.failedPreviousStep:
            # then throw a dice!!! We only want 1% of the success logs
            if random.randint(1, 100) != 1:
                logging.info("Success job! Not saving its logs to CERN EOS recent area.")
                return
            else:
                logging.info("Lucky success job! Saving its logs to CERN EOS recent area.")
        else:
            logging.info("Failed job! Saving its logs to CERN EOS recent area")

        numRetries = 0
        retryPauseT = 0
        if not useNewStageOutCode:
            # old style
            eosmanager = StageOutMgr.StageOutMgr(**eosStageOutParams)
            eosmanager.numberOfRetries = numRetries
            eosmanager.retryPauseTime = retryPauseT
        else:
            # new style
            logging.info("LOGARCHIVE IS USING NEW STAGEOUT CODE For EOS Copy")
            eosmanager = WMCore.Storage.FileManager.StageOutMgr(
                    retryPauseTime=retryPauseT,
                    numberOfRetries=numRetries,
                    **eosStageOutParams)

        eosFileInfo = {'LFN': self.getEOSLogLFN(),
                       'PFN': tarBallLocation,
                       'PNN': None,
                       'GUID': None
                       }

        msg = "Writing logs to CERN EOS recent with retries: %s and retry pause: %s"
        logging.info(msg, eosmanager.numberOfRetries, eosmanager.retryPauseTime)
        try:
            eosmanager(eosFileInfo)
            eosServerPrefix = eosStageOutParams['lfn-prefix'].replace("root://eoscms.cern.ch//eos/cms",
                                                                      "https://eoscmsweb.cern.ch/eos/cms")
            self.report.setLogURL(eosServerPrefix + eosFileInfo['LFN'])
            self.saveReport()
        except Alarm:
            msg = "Indefinite hang during stageOut of logArchive to EOS, ignoring it"
            logging.error(msg)
        except Exception as ex:
            logging.exception("EOS copy failed, lfn: %s. Error: %s", eosFileInfo['LFN'], str(ex))

        return

    def execute(self, emulator=None):
        """
        _execute_

        """
        # Are we using emulators again?
        if emulator is not None:
            return emulator.emulate(self.step, self.job)

        logging.info("Steps.Executors.%s.execute called", self.__class__.__name__)

        overrides = {}
        if hasattr(self.step, 'override'):
            overrides = self.step.override.dictionary_()
        logging.info("Using the following overrides: %s ", overrides)
        # Find alternate stageout location
        self.altLFN = overrides.get('altLFN', None)
        self.failedPreviousStep = overrides.get('previousCmsRunFailure', False)

        logging.info("Step configuration is: %s", self.step)
        # Wait timeout for stageOut
        waitTime = overrides.get('waitTime', 3600 + (self.step.retryDelay * self.step.retryCount))

        matchFiles = [
            ".log$",  # matches the scram, wmagent and cmsRun logs
            "FrameworkJobReport.xml",
            "Report.pkl",
            "^PSet.py$",
            "^PSet.pkl$",
            "_condor_std*",  # condor wrapper logs at the pilot top level
        ]

        ignoredDirs = ['Utils', 'WMCore', 'WMSandbox']

        # Okay, we need a stageOut Manager
        useNewStageOutCode = False
        if getattr(self.step, 'newStageout', False) or \
                ('newStageOut' in overrides and overrides.get('newStageOut')):
            useNewStageOutCode = True
        if not useNewStageOutCode:
            # old style
            manager = StageOutMgr.StageOutMgr(**overrides)
            manager.numberOfRetries = self.step.retryCount
            manager.retryPauseTime = self.step.retryDelay
        else:
            # new style
            logging.info("LOGARCHIVE IS USING NEW STAGEOUT CODE")
            manager = WMCore.Storage.FileManager.StageOutMgr(retryPauseTime=self.step.retryDelay,
                                                             numberOfRetries=self.step.retryCount,
                                                             **overrides)

        # Now we need to find all the reports
        # The log search follows this structure: ~pilotArea/jobArea/WMTaskSpaceArea/StepsArea
        # Start looking at the pilot scratch area first, such that we find the condor logs
        # Then look at the job area in order to find the wmagentJob log
        # Finally, at the taskspace area to find the cmsRun/FWJR/PSet files
        pilotScratchDir = os.path.join(self.stepSpace.taskSpace.location, '../../')
        logFilesToArchive = self.findFilesInDirectory(pilotScratchDir, matchFiles, ignoredDirs)

        # What if it's empty?
        if len(logFilesToArchive) == 0:
            msg = "Couldn't find any log files in the job"
            logging.error(msg)
            return logFilesToArchive

        # Now that we've gone through all the steps, we have to tar it out
        tarName = 'logArchive.tar.gz'
        tarBallLocation = os.path.join(self.stepSpace.location, tarName)
        with tarfile.open(tarBallLocation, 'w:gz') as tarBall:
            for fName in logFilesToArchive:
                altName = fName.replace(pilotScratchDir, '', 1)
                tarBall.add(name=fName, arcname=altName)

        fileInfo = {'LFN': self.getLFN(tarName),
                    'PFN': tarBallLocation,
                    'PNN': None,
                    'GUID': None
                    }
        signal.signal(signal.SIGALRM, alarmHandler)
        signal.alarm(waitTime)

        try:
            manager(fileInfo)
            self.report.addOutputModule(moduleName="logArchive")
            (adler32, cksum) = calculateChecksums(tarBallLocation)
            reportFile = {"lfn": fileInfo["LFN"], "pfn": fileInfo["PFN"],
                          "location": fileInfo["PNN"], "module_label": "logArchive",
                          "events": 0, "size": 0, "merged": False,
                          "checksums": {'adler32': adler32, 'cksum': cksum}}
            self.report.addOutputFile(outputModule="logArchive", aFile=reportFile)
        except Alarm:
            msg = "Indefinite hang during stageOut of logArchive"
            logging.error(msg)
            self.report.addError(self.stepName, 60404, "LogArchiveTimeout", msg)
            self.saveReport()
            raise WMExecutionFailure(60404, "LogArchiveTimeout", msg)
        except WMException as ex:
            self.report.addError(self.stepName, 60307, "LogArchiveFailure", str(ex))
            self.saveReport()
            raise ex
        except Exception as ex:
            self.report.addError(self.stepName, 60405, "LogArchiveFailure", str(ex))
            self.saveReport()
            msg = "Failure in transferring logArchive tarball\n"
            logging.exception(msg)
            raise WMException("LogArchiveFailure", message=str(ex))
        signal.alarm(0)

        signal.alarm(waitTime)
        self.sendLogToEOS(overrides, tarBallLocation, useNewStageOutCode)
        signal.alarm(0)

        return

    def post(self, emulator=None):
        """
        _post_

        Post execution checkpointing

        """
        # Another emulator check
        if emulator is not None:
            return emulator.emulatePost(self.step)

        logging.info("Steps.Executors.%s.post called", self.__class__.__name__)
        return None

    def findFilesInDirectory(self, dirName, matchFiles, ignoredDirs):
        """
        _findFilesInDirectory_

        Given a directory - usually the task space directory - it matches
        the files to the specified patterns.
        """
        logFiles = []
        for f in os.listdir(dirName):
            if f in ignoredDirs:
                continue

            fileName = os.path.join(dirName, f)
            if os.path.isdir(fileName):
                # If directory, use recursion
                logFiles.extend(self.findFilesInDirectory(fileName, matchFiles, ignoredDirs))
            else:
                for match in matchFiles:
                    # Go through each type of match
                    if re.search(match, f):
                        logFiles.append(fileName)
                        break

        # Return final files
        return logFiles

    def getLFN(self, tarName):
        """
        getLFN

        LFNs are messy, do the messy stuff here
        """

        if hasattr(self.task.data, 'RequestTime'):
            reqTime = time.gmtime(int(self.task.data.RequestTime))
        else:
            reqTime = time.gmtime()

        year, month, day = reqTime[:3]

        LFN = "%s/logs/prod/%s/%s/%s%s/%s/%i/%s-%i-%s" % \
              (self.task.taskLogBaseLFN(), year, month, day, self.task.getPathName(),
               lfnGroup(self.job), self.job.get('retry_count', 0), self.job["name"],
               self.job.get('retry_count', 0),
               tarName)

        if self.altLFN:
            LFN = "%s/%s/%s/%i/%s-%i-%s" % (self.altLFN, self.task.getPathName(),
                                            lfnGroup(self.job), self.job.get('retry_count', 0), self.job["name"],
                                            self.job.get('retry_count', 0), tarName)

        return LFN

    def getEOSLogLFN(self):
        """
        getEOSLogLFN

        LFNs are messy, do the messy stuff here
         /eos/cms/store/logs/prod/recent/vlimant_task_TOP-RunIIFall17wmLHEGS-00073__v1_T_180219_102942_9978/TOP-RunIIFall17DRPremix-00053_0/vocms0251.cern.ch-3803-0.log.tar.gz
        """
        taskPath = self.task.getPathName().split("/")
        requestName = taskPath[1]
        lastTask = taskPath[-1]
        LFN = "/%s/%s/%s-%s-%s-log.tar.gz" % (requestName, lastTask, self.job.get("agentName", 'NA'),
                                              self.job["id"], self.job.get('retry_count', 0))

        return LFN



from __future__ import absolute_import
from builtins import range, object

from .Globals import GlobalParams
from . import Globals

class NoDatasetError(Exception):
    """Standard error baseclass"""
    def __init__(self, error):
        Exception.__init__(self, error)
        self.msg = 'NoDatasetError'
        self.error = error

class DataBlockGenerator(object):

    def _blockGenerator(self, dataset):
        if dataset.startswith('/' + Globals.NOT_EXIST_DATASET):
            raise NoDatasetError("no dataset")
        blocks = []
        numOfEvents = GlobalParams.numOfFilesPerBlock() * GlobalParams.numOfEventsPerFile()
        for i in range(GlobalParams.numOfBlocksPerDataset()):
            blockName = "%s#%s" % (dataset, i+1)
            size = GlobalParams.numOfFilesPerBlock() * GlobalParams.sizeOfFile()

            blocks.append({'Name' : blockName,
                           'NumberOfEvents' : numOfEvents,
                           'NumberOfFiles' : GlobalParams.numOfFilesPerBlock(),
                           'NumberOfLumis' : GlobalParams.numOfLumisPerBlock(),
                           'Size' : size,
                           'Parents' : ()}
                           )
        return blocks

    def getParentBlock(self, block, numberOfParents = 1):
        blocks = []
        numOfEvents = GlobalParams.numOfFilesPerBlock() * GlobalParams.numOfEventsPerFile()
        for i in range(numberOfParents):
            dataset, blockname = block.split('#') # append parent block id to tier
            blockName = "%s_parent_%s#%s" % (dataset, i+1, blockname)
            size = GlobalParams.numOfFilesPerBlock() * GlobalParams.sizeOfFile()

            blocks.append({'Name' : blockName,
                           'NumberOfEvents' : numOfEvents,
                           'NumberOfFiles' : GlobalParams.numOfFilesPerBlock(),
                           'NumberOfLumis' : GlobalParams.numOfLumisPerBlock(),
                           'Size' : size,
                           'PhEDExNodeList' : self.getLocation(blockName),
                           'Parents' : ()}
                           )
        return blocks

    def _fileGenerator(self, blockName, parentFlag):

        files = []

        for fileID in range(GlobalParams.numOfFilesPerBlock()):

            fileName =  "/store/data%s/file%s" % (blockName, fileID)
            #Not sure why fileName is unicode - change to str
            #FIXME - python3: this will likely not be necessary anymore in py3
            fileName = str(fileName)
            parentFileName = "/store/data%s_parent/file%s_parent" % (blockName, fileID)
            #Not sure why fileName is unicode - change to str
            #FIXME - python3: this will likely not be necessary anymore in py3
            parentFileName = str(parentFileName)

            if parentFlag:
                parentList = [self._createDBSFile(blockName, {'LogicalFileName':parentFileName})]
            else:
                parentList = []
            dbsFile = {'LogicalFileName': fileName,
                       'ParentList' : parentList,
                      }
            files.append(self._createDBSFile(blockName, dbsFile))
        return files

    def _createDBSFile(self, blockName, dbsFile = {}):
        run =  GlobalParams.getRunNumberForBlock(blockName)
        defaultDBSFile = {'Checksum': "123456",
                          'NumberOfEvents': GlobalParams.numOfEventsPerFile(),
                          'FileSize': GlobalParams.sizeOfFile(),
                          'ParentList': [],
                          # assign run and lumi numbers in appropriate range: note run numbers
                          #  for successive blocks may overlap if numOfRunsPerFile() > 1
                          'LumiList': [{'RunNumber': GlobalParams.numOfRunsPerFile(),
                                        'LumiSectionNumber': list(range(1, GlobalParams.numOfLumisPerBlock()+1))}]
                          }
        defaultDBSFile.update(dbsFile)
        return defaultDBSFile

    def getBlocks(self, dataset):
        try:
            return self._blockGenerator(dataset)
        except NoDatasetError:
            return []

    def getLocation(self, block):
        return Globals.getSites(block)

    def getDatasetName(self, block):
        return block.split('#')[0]



"""
Unit tests for the WMCore/MicroService/DataStructs/Workflow.py module
"""
from __future__ import division, print_function

import unittest

from Utils.PythonVersion import PY3

from WMCore.MicroService.MSTransferor.DataStructs.Workflow import Workflow


class WorkflowTest(unittest.TestCase):
    """
    Test the very basic functionality of the Workflow module
    """

    def setUp(self):
        if PY3:
            self.assertItemsEqual = self.assertCountEqual

    def testCampaignMap(self):
        """
        Test setting the data campaign map for a TaskChain-like request
        """
        parentDset = "/any/parent-dataset/tier"
        tChainSpec = {"RequestType": "TaskChain",
                      "TaskChain": 4,
                      "Campaign": "top-campaign",
                      "RequestName": "whatever_name",
                      "DbsUrl": "https://cmsweb.cern.ch/dbs/prod/global/DBSReader/",
                      "Task1": {"InputDataset": "/task1/input-dataset/tier",
                                "Campaign": "task1-campaign",
                                "IncludeParents": True},
                      "Task2": {"DataPileup": "/task2/data-pileup/tier"},
                      "Task3": {"MCPileup": "/task3/mc-pileup/tier",
                                "Campaign": "task3-campaign"},
                      "Task4": {"MCPileup": "/task3/mc-pileup/tier",
                                "Campaign": "task3-campaign"},
                      }
        wflow = Workflow(tChainSpec['RequestName'], tChainSpec)
        self.assertEqual(len(wflow.getDataCampaignMap()), 3)
        self.assertEqual(wflow.getDbsUrl(), "https://cmsweb-prod.cern.ch/dbs/prod/global/DBSReader")
        for dataIn in wflow.getDataCampaignMap():
            if dataIn['type'] == "primary":
                self.assertItemsEqual(dataIn, {"type": "primary", "campaign": tChainSpec['Task1']['Campaign'],
                                               "name": tChainSpec['Task1']['InputDataset']})
            elif dataIn['name'] == tChainSpec['Task2']['DataPileup']:
                self.assertItemsEqual(dataIn, {"type": "secondary", "campaign": tChainSpec['Campaign'],
                                               "name": tChainSpec['Task2']['DataPileup']})
            else:
                self.assertItemsEqual(dataIn, {"type": "secondary", "campaign": tChainSpec['Task3']['Campaign'],
                                               "name": tChainSpec['Task3']['MCPileup']})

        wflow.setParentDataset(parentDset)
        self.assertEqual(wflow.getParentDataset(), parentDset)
        self.assertEqual(len(wflow.getDataCampaignMap()), 4)
        for dataIn in wflow.getDataCampaignMap():
            if dataIn['type'] == "parent":
                self.assertItemsEqual(dataIn, {"type": "parent", "campaign": tChainSpec['Task1']['Campaign'],
                                               "name": parentDset})

    def testReRecoWflow(self):
        """
        Test loading a ReReco like request into Workflow
        """
        parentDset = "/rereco/parent-dataset/tier"
        rerecoSpec = {"RequestType": "ReReco",
                      "InputDataset": "/rereco/input-dataset/tier",
                      "Campaign": "any-campaign",
                      "RequestName": "whatever_name",
                      "DbsUrl": "a_dbs_url",
                      "SiteWhitelist": ["CERN", "FNAL", "DESY"],
                      "SiteBlacklist": ["FNAL"]}
        wflow = Workflow(rerecoSpec['RequestName'], rerecoSpec)
        self.assertEqual(wflow.getName(), rerecoSpec['RequestName'])
        self.assertEqual(wflow.getDbsUrl(), rerecoSpec['DbsUrl'])
        self.assertItemsEqual(wflow.getSitelist(), ["CERN", "DESY"])
        self.assertItemsEqual(wflow.getCampaigns(), [rerecoSpec["Campaign"]])
        self.assertEqual(wflow.getInputDataset(), rerecoSpec["InputDataset"])
        self.assertItemsEqual(wflow.getPileupDatasets(), set())
        self.assertFalse(wflow.hasParents())
        self.assertEqual(wflow.getParentDataset(), "")
        self.assertEqual(wflow.getPrimaryBlocks(), {})
        self.assertEqual(wflow.getSecondarySummary(), {})
        self.assertEqual(wflow.getParentBlocks(), {})
        self.assertEqual(wflow._getValue("NoKey"), None)
        self.assertEqual(len(wflow.getDataCampaignMap()), 1)

        wflow.setParentDataset(parentDset)
        self.assertEqual(wflow.getParentDataset(), parentDset)
        self.assertEqual(len(wflow.getDataCampaignMap()), 2)

    def testTaskChainWflow(self):
        """
        Test loading a TaskChain like request into Workflow
        """
        tChainSpec = {"RequestType": "TaskChain",
                      "TaskChain": 3,
                      "Campaign": "top-campaign",
                      "RequestName": "whatever_name",
                      "DbsUrl": "a_dbs_url",
                      "SiteWhitelist": ["CERN", "FNAL", "DESY"],
                      "SiteBlacklist": [],
                      "Task1": {"InputDataset": "/task1/input-dataset/tier",
                                "MCPileup": "/task1/mc-pileup/tier",
                                "Campaign": "task1-campaign"},
                      "Task2": {"DataPileup": "/task2/data-pileup/tier",
                                "Campaign": "task2-campaign"},
                      "Task3": {"MCPileup": "/task1/mc-pileup/tier",
                                "Campaign": "task3-campaign"},
                      }
        wflow = Workflow(tChainSpec['RequestName'], tChainSpec)
        self.assertEqual(wflow.getName(), tChainSpec['RequestName'])
        self.assertEqual(wflow.getDbsUrl(), tChainSpec['DbsUrl'])
        self.assertItemsEqual(wflow.getSitelist(), tChainSpec['SiteWhitelist'])
        campaigns = ["%s-campaign" % c for c in {"task1", "task2", "task3"}]
        self.assertItemsEqual(wflow.getCampaigns(), campaigns)
        self.assertEqual(wflow.getInputDataset(), tChainSpec['Task1']['InputDataset'])
        pileups = [tChainSpec['Task1']['MCPileup'], tChainSpec['Task2']['DataPileup']]
        self.assertItemsEqual(wflow.getPileupDatasets(), pileups)
        self.assertFalse(wflow.hasParents())
        self.assertEqual(wflow.getParentDataset(), "")
        self.assertEqual(wflow.getPrimaryBlocks(), {})
        self.assertEqual(wflow.getSecondarySummary(), {})
        self.assertEqual(wflow.getParentBlocks(), {})
        self.assertEqual(wflow._getValue("NoKey"), None)
        self.assertEqual(len(wflow.getDataCampaignMap()), 3)

    def testStepChainWflow(self):
        """
        Test loading a StepChain like request into Workflow
        """
        tChainSpec = {"RequestType": "StepChain",
                      "StepChain": 3,
                      "Campaign": "top-campaign",
                      "RequestName": "whatever_name",
                      "DbsUrl": "a_dbs_url",
                      "SiteWhitelist": ["CERN", "FNAL", "DESY"],
                      "SiteBlacklist": [],
                      "Step1": {"InputDataset": "/step1/input-dataset/tier",
                                "MCPileup": "/step1/mc-pileup/tier",
                                "Campaign": "step1-campaign"},
                      "Step2": {"DataPileup": "/step2/data-pileup/tier",
                                "Campaign": "step2-campaign"},
                      "Step3": {"MCPileup": "/step1/mc-pileup/tier",
                                "Campaign": "step3-campaign"},
                      }
        wflow = Workflow(tChainSpec['RequestName'], tChainSpec)
        self.assertEqual(wflow.getName(), tChainSpec['RequestName'])
        self.assertEqual(wflow.getDbsUrl(), tChainSpec['DbsUrl'])
        self.assertItemsEqual(wflow.getSitelist(), tChainSpec['SiteWhitelist'])
        campaigns = ["%s-campaign" % c for c in {"step1", "step2", "step3"}]
        self.assertItemsEqual(wflow.getCampaigns(), campaigns)
        self.assertEqual(wflow.getInputDataset(), tChainSpec['Step1']['InputDataset'])
        pileups = [tChainSpec['Step1']['MCPileup'], tChainSpec['Step2']['DataPileup']]
        self.assertItemsEqual(wflow.getPileupDatasets(), pileups)
        self.assertFalse(wflow.hasParents())
        self.assertEqual(wflow.getParentDataset(), "")
        self.assertEqual(wflow.getPrimaryBlocks(), {})
        self.assertEqual(wflow.getSecondarySummary(), {})
        self.assertEqual(wflow.getParentBlocks(), {})
        self.assertEqual(wflow._getValue("NoKey"), None)
        self.assertEqual(len(wflow.getDataCampaignMap()), 3)

    def testResubmission(self):
        """
        Test loading a Resubmission like request into Workflow
        """
        rerecoSpec = {"RequestType": "Resubmission",
                      "InputDataset": "/rereco/input-dataset/tier",
                      "Campaign": "any-campaign",
                      "RequestName": "whatever_name",
                      "DbsUrl": "a_dbs_url",
                      "SiteWhitelist": ["CERN", "FNAL", "DESY"],
                      "SiteBlacklist": ["FNAL"]}
        wflow = Workflow(rerecoSpec['RequestName'], rerecoSpec)
        # we do not set any map for Resubmission workflows
        self.assertEqual(wflow.getDataCampaignMap(), [])

    def testGetParam(self):
        """
        Test the `getReqParam` method
        """
        tChainSpec = {"RequestType": "StepChain",
                      "StepChain": 1,
                      "Campaign": "top-campaign",
                      "RequestName": "whatever_name",
                      "DbsUrl": "a_dbs_url",
                      "TrustSitelists": True,
                      "SiteWhitelist": ["CERN", "FNAL", "DESY"],
                      "SiteBlacklist": [],
                      "Step1": {"InputDataset": "/step1/input-dataset/tier",
                                "MCPileup": "/step1/mc-pileup/tier",
                                "Campaign": "step1-campaign"},
                      }
        wflow = Workflow(tChainSpec['RequestName'], tChainSpec)
        self.assertTrue(wflow.getReqParam("TrustSitelists"))
        self.assertIsNone(wflow.getReqParam("MCPileup"))
        self.assertEqual(wflow.getReqParam("RequestType"), wflow.getReqType())
        self.assertEqual(wflow.getReqParam("RequestName"), wflow.getName())

    def testComparison(self):
        """
        Perform basic operations over Workflow objects
        """
        wflow1 = Workflow("workflow_1", {"RequestType": "StepChain", "DbsUrl": "a_dbs_url"})
        wflow2 = Workflow("workflow_2", {"RequestType": "TaskChain", "DbsUrl": "a_dbs_url"})
        wflow3 = Workflow("workflow_3", {"RequestType": "ReReco", "DbsUrl": "a_dbs_url"})
        wflow4 = Workflow("workflow_4", {"RequestType": "StepChain", "DbsUrl": "a_dbs_url"})
        listWflows = [wflow1, wflow2, wflow3, wflow4]

        self.assertNotEqual(wflow1, wflow4)

        badWflows = [wflow3, wflow3]
        self.assertEqual(len(listWflows), 4)
        self.assertEqual(len(badWflows), 2)
        for wflow in set(badWflows):
            listWflows.remove(wflow)
        self.assertEqual(len(listWflows), 3)
        self.assertEqual(len(badWflows), 2)

    def testParentageRelationship(self):
        """
        Test methods related to the primary and parent datasets and blocks
        """
        primDict = {"block_A": {"blockSize": 1, "locations": ["Site_A"]},
                    "block_B": {"blockSize": 2, "locations": ["Site_B"]}}
        parentDict = {"parent_A": {"blockSize": 11, "locations": ["Site_A"]},
                      "parent_B": {"blockSize": 12, "locations": ["Site_B"]},
                      "parent_C": {"blockSize": 13, "locations": ["Site_A", "Site_B"]}}
        parentage = {"block_A": ["parent_B", "parent_D"],  # parent_D has no replicas!
                     "block_B": ["parent_A", "parent_C"]}
        wflow = Workflow("workflow_1", {"RequestType": "TaskChain",
                                        "InputDataset": "Dataset_name_XXX",
                                        "DbsUrl": "a_dbs_url",
                                        "IncludeParents": True})

        self.assertEqual(wflow.getParentDataset(), "")
        wflow.setParentDataset("Parent_dataset_XXX")
        self.assertEqual(wflow.getParentDataset(), "Parent_dataset_XXX")

        self.assertEqual(wflow.getPrimaryBlocks(), {})
        wflow.setPrimaryBlocks(primDict)
        self.assertItemsEqual(list(wflow.getPrimaryBlocks()), ["block_A", "block_B"])

        self.assertEqual(wflow.getParentBlocks(), {})
        wflow.setParentBlocks(parentDict)
        self.assertItemsEqual(list(wflow.getParentBlocks()), ["parent_A", "parent_B", "parent_C"])

        self.assertEqual(wflow.getChildToParentBlocks(), {})
        wflow.setChildToParentBlocks(parentage)
        self.assertItemsEqual(wflow.getChildToParentBlocks(), parentage)

    def testGetInputData(self):
        """
        Test the `getInputData` method both with input primary and
        parent blocks
        """
        primDict = {"block_A": {"blockSize": 1, "locations": ["Site_A"]},
                    "block_B": {"blockSize": 2, "locations": ["Site_B"]}}
        parentDict = {"parent_A": {"blockSize": 11, "locations": ["Site_A"]},
                      "parent_B": {"blockSize": 12, "locations": ["Site_B"]},
                      "parent_C": {"blockSize": 13, "locations": ["Site_A", "Site_B"]}}
        parentage = {"block_A": ["parent_B", "parent_D"],  # parent_D has no replicas!
                     "block_B": ["parent_A", "parent_C"]}
        wflow = Workflow("workflow_1", {"RequestType": "TaskChain",
                                        "InputDataset": "Dataset_name_XXX",
                                        "DbsUrl": "a_dbs_url"})

        wflow.setPrimaryBlocks(primDict)
        blockChunks, sizeChunks = wflow.getInputData()
        self.assertEqual(len(blockChunks), 2)
        self.assertItemsEqual(blockChunks, {"block_A", "block_B"})
        self.assertEqual(sizeChunks, 3)

        # now set a parent
        wflow.setParentDataset("Parent_dataset_XXX")
        wflow.setPrimaryBlocks(primDict)
        wflow.setParentBlocks(parentDict)
        wflow.setChildToParentBlocks(parentage)

        blockChunks, sizeChunks = wflow.getInputData()
        self.assertEqual(len(blockChunks), 5)
        self.assertItemsEqual(blockChunks, {"block_A", "block_B", "parent_A", "parent_B", "parent_C"})
        self.assertEqual(sizeChunks, 39)

    def testGetWorkflowGroup(self):
        """
        Test the `getWorkflowGroup` method functionality
        """
        requestTypes = ("StepChain", "TaskChain", "ReReco")
        for wflowType in requestTypes:
            wflowObj = Workflow("wflow_test", {"RequestType": wflowType, "DbsUrl": "a_dbs_url"})
            self.assertEqual(wflowObj.getWorkflowGroup(), "production")

            wflowObj = Workflow("wflow_test", {"RequestType": wflowType, "SubRequestType": "ReDigi",
                                               "DbsUrl": "a_dbs_url"})
            self.assertEqual(wflowObj.getWorkflowGroup(), "production")

        for wflowType in requestTypes:
            wflowObj = Workflow("wflow_test", {"RequestType": wflowType, "SubRequestType": "RelVal",
                                               "DbsUrl": "a_dbs_url"})
            self.assertEqual(wflowObj.getWorkflowGroup(), "relval")

    def testGetRucioGrouping(self):
        """
        Test the `getRucioGrouping` method, which is supposed to return
        a basic string with the Rucio grouping for this template (static
        output).
        """
        parentDset = "/rereco/parent-dataset/tier"
        rerecoSpec = {"RequestType": "ReReco",
                      "InputDataset": "/rereco/input-dataset/tier",
                      "Campaign": "any-campaign",
                      "RequestName": "whatever_name",
                      "DbsUrl": "a_dbs_url",
                      "SiteWhitelist": ["CERN", "FNAL", "DESY"],
                      "SiteBlacklist": ["FNAL"]}
        wflow = Workflow(rerecoSpec['RequestName'], rerecoSpec)
        self.assertEqual(wflow.getRucioGrouping(), "DATASET")

        wflow.setParentDataset(parentDset)
        self.assertEqual(wflow.getRucioGrouping(), "ALL")

    def testGetReplicaCopies(self):
        """
        Test the `getReplicaCopies` method, which is supposed to return
        an integer with the number of copies that a rule has to request
        """
        rerecoSpec = {"RequestType": "ReReco",
                      "InputDataset": "/rereco/input-dataset/tier",
                      "Campaign": "any-campaign",
                      "RequestName": "whatever_name",
                      "DbsUrl": "a_dbs_url",
                      "SiteWhitelist": ["CERN", "FNAL", "DESY"],
                      "SiteBlacklist": ["FNAL"]}
        wflow = Workflow(rerecoSpec['RequestName'], rerecoSpec)
        self.assertEqual(wflow.getReplicaCopies(), 1)


if __name__ == '__main__':
    unittest.main()



"""
Created on Jun 18, 2009
@author: meloam
Modified on Nov. 7, 2023 by Duong Nguyen
"""
import logging
import os
import unittest

from WMCore_t.Storage_t.DeleteMgr_t import DeleteMgrTest
from future.utils import viewitems

from WMCore.Storage.Registry import retrieveStageOutImpl
from WMCore.Storage.SiteLocalConfig import stageOutStr
from WMCore.Storage.StageOutError import StageOutFailure
from WMCore.Storage.StageOutMgr import StageOutMgr, searchRFC
from WMCore.WMBase import getTestBase


class StageOutMgrTest(StageOutMgr):
    def stageOut(self, lfn, localPfn, checksums, stageOut_rfc=None):
        """
        Given the lfn and a pair of stage out and corresponding Rucio file catalog, stageOut_rfc, or override configuration invoke the stage out
        If use override configuration self.overrideConf should contain:
        command - the stage out impl plugin name to be used
        option - the option values to be passed to that command (None is allowed)
        lfn-prefix - the LFN prefix to generate the PFN
        phedex-node - the Name of the PNN to which the file is being xferred
        :param lfn: logical file name 
        :param localPfn: physical file name of file at local location (source) that will be staged out to another location (destination)
        :param checksums: check sum of file
        :param stageOut_rfc: a pair of stage out and corresponding Rucio file catalog
        """

        if not self.override:
            if not stageOut_rfc:
                msg = "Can not perform stage out for this lfn because of missing stage out information (stageOut_rfc is None or empty): \n %s" % lfn
                raise StageOutFailure(msg, LFN=lfn)
            command = stageOut_rfc[0]['command']
            options = stageOut_rfc[0]['option']
            pfn = searchRFC(stageOut_rfc[1], lfn)
            protocol = stageOut_rfc[1].preferredProtocol
            if pfn == None:
                msg = "Unable to match lfn to pfn: \n  %s" % lfn
                raise StageOutFailure(msg, LFN=lfn, StageOut=stageOutStr(stageOut_rfc[0]))
            try:
                impl = retrieveStageOutImpl(command)
            except Exception as ex:
                msg = "Unable to retrieve impl for local stage out:\n"
                msg += "Error retrieving StageOutImpl for command named: %s\n" % (
                    command,)
                raise StageOutFailure(msg, Command=command,
                                      LFN=lfn, ExceptionDetail=str(ex))
            impl.numRetries = self.numberOfRetries
            impl.retryPause = self.retryPauseTime

            try:
                pass
            except Exception as ex:
                msg = "Failure for stage out:\n"
                msg += str(ex)
                try:
                    import traceback
                    msg += traceback.format_exc()
                except AttributeError:
                    msg += "Traceback unavailable\n"
                raise StageOutFailure(msg, Command=command, Protocol=protocol,
                                      LFN=lfn, InputPFN=localPfn, TargetPFN=pfn)
            return pfn

        else:
            if self.overrideConf['lfn-prefix'] is None:
                msg = "Unable to match lfn to pfn during stage out because override lfn-prefix is None: \n %s" % lfn
                raise StageOutFailure(msg, LFN=lfn)

            pfn = "%s%s" % (self.overrideConf['lfn-prefix'], lfn)

            try:
                impl = retrieveStageOutImpl(self.overrideConf['command'])
            except Exception as ex:
                msg = "Unable to retrieve impl for override stage out:\n"
                msg += "Error retrieving StageOutImpl for command named: "
                msg += "%s\n" % self.overrideConf['command']
                raise StageOutFailure(msg, Command=self.overrideConf['command'],
                                      LFN=lfn, ExceptionDetail=str(ex))

            impl.numRetries = self.numberOfRetries
            impl.retryPause = self.retryPauseTime

            try:
                pass
            except Exception as ex:
                msg = "Failure for override stage out:\n"
                msg += str(ex)
                raise StageOutFailure(msg, Command=self.overrideConf['command'],
                                      LFN=lfn, InputPFN=localPfn, TargetPFN=pfn)

            return pfn

    def cleanSuccessfulStageOuts(self):
        """
        In the event of a failed stage out, this method can be called to cleanup the
        files that may have previously been staged out so that the job ends in a clear state
        of failure, rather than a partial success
        """

        for lfn, fileInfo in viewitems(self.completedFiles):
            pfn = fileInfo['PFN']
            command = fileInfo['StageOutCommand']
            msg = "Cleaning out file: %s\n" % lfn
            msg += "Removing PFN: %s" % pfn
            msg += "Using command implementation: %s\n" % command
            logging.info(msg)
            # Need to use DeleteMgrTest so that the actual deletion will not proceed
            delManager = DeleteMgrTest(**self.overrideConf)
            try:
                delManager.deletePFN(pfn, lfn, command)
            except StageOutFailure as ex:
                msg = "Failed to cleanup staged out file after error:"
                msg += " %s\n%s" % (lfn, str(ex))
                logging.error(msg)


class StageOutMgrUnitTest(unittest.TestCase):

    def setUp(self):
        # shut up SiteLocalConfig
        os.putenv('CMS_PATH', os.getcwd())
        os.putenv('SITECONFIG_PATH', os.getcwd())

    def testStageOutMgr(self):
        configFileName = os.path.join(getTestBase(),
                                      "WMCore_t/Storage_t/T1_DE_KIT/JobConfig",
                                      "site-local-config.xml")
        os.environ["WMAGENT_SITE_CONFIG_OVERRIDE"] = configFileName
        os.environ['SITECONFIG_PATH'] = os.path.join(getTestBase(),
                                                     "WMCore_t/Storage_t",
                                                     "T1_DE_KIT")
        os.system(
            'cp $SITECONFIG_PATH/JobConfig/site-local-config-testStageOut-T1_DE_KIT.xml $SITECONFIG_PATH/JobConfig/site-local-config.xml')
        stageOutMgr = StageOutMgrTest()
        fileToStage = {'LFN': '/store/abc/xyz.root', 'PFN': ''}
        stageOutMgr(fileToStage)
        assert fileToStage[
                   'PFN'] == "davs://cmswebdav-kit-disk.gridka.de:2880/pnfs/gridka.de/cms/disk-only/store/abc/xyz.root"
        stageOutMgr.cleanSuccessfulStageOuts()

        # test override
        stageOutMgr_override = StageOutMgrTest(
            **{"command": "gfal2", "phedex-node": "T1_US_FNAL_Disk", "lfn-prefix": "root://abc/xyz"})
        fileToStage = {'LFN': '/store/abc/xyz.root', 'PFN': ''}
        stageOutMgr_override(fileToStage)
        assert fileToStage['PFN'] == "root://abc/xyz/store/abc/xyz.root"
        stageOutMgr_override.cleanSuccessfulStageOuts()

        # test chained rules
        os.system(
            'cp $SITECONFIG_PATH/JobConfig/site-local-config-testStageOut-chainedRules.xml $SITECONFIG_PATH/JobConfig/site-local-config.xml')
        stageOutMgr = StageOutMgrTest()
        fileToStage = {'LFN': '/store/abc/xyz.root', 'PFN': ''}
        stageOutMgr(fileToStage)
        assert fileToStage[
                   'PFN'] == "davs://cmswebdav-kit-tape.gridka.de:2880/pnfs/gridka.de/cms/tape/store/abc/xyz.root"
        stageOutMgr.cleanSuccessfulStageOuts()

        # test stage-out to another site, T2_DE_DESY
        os.system(
            'cp $SITECONFIG_PATH/JobConfig/site-local-config-testStageOut-T2_DE_DESY.xml $SITECONFIG_PATH/JobConfig/site-local-config.xml')
        stageOutMgr = StageOutMgrTest()
        fileToStage = {'LFN': '/store/abc/xyz.root', 'PFN': ''}
        stageOutMgr(fileToStage)
        assert fileToStage[
                   'PFN'] == "davs://dcache-cms-webdav-wan.desy.de:2880/pnfs/desy.de/cms/tier2/store/abc/xyz.root"
        stageOutMgr.cleanSuccessfulStageOuts()

        # test subsite
        configFileName = os.path.join(getTestBase(),
                                      "WMCore_t/Storage_t/T1_DE_KIT/KIT-T3/JobConfig",
                                      "site-local-config.xml")
        os.environ["WMAGENT_SITE_CONFIG_OVERRIDE"] = configFileName
        os.environ['SITECONFIG_PATH'] = os.path.join(getTestBase(),
                                                     "WMCore_t/Storage_t",
                                                     "T1_DE_KIT/KIT-T3")
        os.system(
            'cp $SITECONFIG_PATH/JobConfig/site-local-config-testStageOut-T1_DE_KIT.xml $SITECONFIG_PATH/JobConfig/site-local-config.xml')
        stageOutMgr = StageOutMgrTest()
        fileToStage = {'LFN': '/store/abc/xyz.root', 'PFN': ''}
        stageOutMgr(fileToStage)
        assert fileToStage[
                   'PFN'] == "davs://cmswebdav-kit-disk.gridka.de:2880/pnfs/gridka.de/cms/disk-only/store/abc/xyz.root"
        stageOutMgr.cleanSuccessfulStageOuts()

        # test subsite with stage-out to another site T2_DE_DESY
        os.system(
            'cp $SITECONFIG_PATH/JobConfig/site-local-config-testStageOut-T2_DE_DESY.xml $SITECONFIG_PATH/JobConfig/site-local-config.xml')
        stageOutMgr = StageOutMgrTest()
        fileToStage = {'LFN': '/store/abc/xyz.root', 'PFN': ''}
        stageOutMgr(fileToStage)
        assert fileToStage[
                   'PFN'] == "davs://dcache-cms-webdav-wan.desy.de:2880/pnfs/desy.de/cms/tier2/store/abc/xyz.root"
        stageOutMgr.cleanSuccessfulStageOuts()

        return


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



#/usr/bin/env python2.4
"""
_DestroyAgentBase_

"""
from __future__ import absolute_import




import threading

from WMCore.Database.DBCreator import DBCreator

from WMCore.WMException import WMException
from WMCore.WMExceptions import WMEXCEPTION
from .CreateAgentBase import CreateAgentBase

class DestroyAgentBase(DBCreator):
    def __init__(self, logger = None, dbi = None):
        """
        _init_

        Call the DBCreator constructor and create the list of required tables.
        """
        myThread = threading.currentThread()

        if logger == None:
            logger = myThread.logger
        if dbi == None:
            dbi = myThread.dbi

        DBCreator.__init__(self, logger, dbi)
        orderedTables = CreateAgentBase.requiredTables[:]
        orderedTables.reverse()
        i = 0
        for requiredTable in orderedTables:
            i += 1
            tableName = requiredTable[2:]
            prefix = str.zfill(str(i), 2)
            self.create[prefix + tableName] = "DROP TABLE %s" % tableName



#!/usr/bin/env python

from __future__ import (division, print_function)

import unittest
import logging
from dbs.apis.dbsClient import DbsApi

from Utils.ExtendedUnitTestCase import ExtendedUnitTestCase
from WMQuality.Emulators.DBSClient.MockDbsApi import MockDbsApi

# a small dataset that should always exist
DATASET = '/HighPileUp/Run2011A-v1/RAW'
BLOCK = '/HighPileUp/Run2011A-v1/RAW#fabf118a-cbbf-11e0-80a9-003048caaace'
FILE_NAMES = [u'/store/data/Commissioning2015/Cosmics/RAW/v1/000/238/545/00000/C47FDF25-2ECF-E411-A8E2-02163E011839.root']


class MockDbsApiTest(ExtendedUnitTestCase):
    """
    Class that can be imported to switch to 'mock'ed versions of
    services.

    """

    def setUp(self):
        self.endpoint = 'https://cmsweb-prod.cern.ch/dbs/prod/global/DBSReader'
        self.realDBS = DbsApi(self.endpoint)
        self.mockDBS = MockDbsApi(self.endpoint)
        return

    def tearDown(self):
        return

    def testDBSApiMembers(self):
        """
        Tests members from DBSApi
        """

        block_with_parent = '/Cosmics/Commissioning2015-PromptReco-v1/RECO#004ac3ba-d09e-11e4-afad-001e67ac06a0'
        # List of members from DBSApi
        members = {'listDataTiers': {},
                   'listDatasets': {'data_tier_name': 'RAW', 'primary_ds_name': 'Jet'},
                   'listFileLumis': {'block_name': BLOCK, 'validFileOnly': 1},
                   # 'listFileLumiArray':{'logical_file_name': FILE_NAMES},
                   'listFileParents': {'block_name': block_with_parent},
                   'listPrimaryDatasets': {'primary_ds_name': 'Jet*'},
                   'listRuns': {'dataset': DATASET},
                   'listFileArray': {'dataset': DATASET, 'detail': True, 'validFileOnly': 1},
                   'listFileSummaries': {'dataset': DATASET, 'validFileOnly': 1},
                   'listBlocks': {'dataset': DATASET, 'detail': True},
                   'listBlockParents': {'block_name': block_with_parent},
                   }

        for member in list(members.keys()):
            # Get from  mock DBS
            args = []
            kwargs = members[member]
            logging.info("Querying API: %s, with parameters: %s", member, members[member])
            real = getattr(self.realDBS, member)(*args, **kwargs)
            mock = getattr(self.mockDBS, member)(*args, **kwargs)

            self.assertContentsEqual(real, mock, "Identity check on member %s failed" % member)

        return


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_CountPhedexNotUploaded_

Oracle implementation of DBSBuffer.CountPhedexNotUploaded
"""

from __future__ import division
from WMComponent.DBS3Buffer.MySQL.CountPhedexNotUploaded import CountPhedexNotUploaded as MySQLCountPhedexNotUploaded

class CountPhedexNotUploaded(MySQLCountPhedexNotUploaded):
    """
    CountPhedexNotUploaded

    """
    pass



#!/usr/bin/env python
"""
WorkQueue splitting by dataset, it creates one single
workqueue element per dataset.

This policy is specifically used by DQMHarvest workflows, which requires
some special handling based on run information. Nonetheless, trying to
make it generic enough that could be used by other spec types.
"""

import logging
from math import ceil
from WMCore import Lexicon
from WMCore.WorkQueue.Policy.Start.StartPolicyInterface import StartPolicyInterface
from WMCore.WorkQueue.WorkQueueExceptions import WorkQueueWMSpecError
from WMCore.WorkQueue.WorkQueueUtils import makeLocationsList


class Dataset(StartPolicyInterface):
    """Split elements into datasets"""

    def __init__(self, **args):
        StartPolicyInterface.__init__(self, **args)
        self.args.setdefault('SliceType', 'NumberOfRuns')
        self.args.setdefault('SliceSize', 1)
        self.lumiType = "NumberOfLumis"
        self.sites = []

    def split(self):
        """Apply policy to spec"""
        work = set() if self.args['SliceType'] == 'NumberOfRuns' else 0
        numFiles = 0
        numEvents = 0
        numLumis = 0
        datasetPath = self.initialTask.getInputDatasetPath()

        # dataset splitting can't have its data selection overridden
        if self.data and list(self.data) != [datasetPath]:
            raise RuntimeError("Can't provide different data to split with")

        blocks = self.validBlocks(self.initialTask, self.dbs())
        if not blocks:
            return

        for block in blocks:
            if self.args['SliceType'] == 'NumberOfRuns':
                work = work.union(block[self.args['SliceType']])
            else:
                work += float(block[self.args['SliceType']])
            numLumis += int(block[self.lumiType])
            numFiles += int(block['NumberOfFiles'])
            numEvents += int(block['NumberOfEvents'])

        if self.args['SliceType'] == 'NumberOfRuns':
            numJobs = ceil(len(work) / float(self.args['SliceSize']))
        else:
            numJobs = ceil(float(work) / float(self.args['SliceSize']))

        # parentage
        parentFlag = True if self.initialTask.parentProcessingFlag() else False

        self.newQueueElement(Inputs={datasetPath: self.data.get(datasetPath, [])},
                             ParentFlag=parentFlag,
                             NumberOfLumis=numLumis,
                             NumberOfFiles=numFiles,
                             NumberOfEvents=numEvents,
                             Jobs=numJobs,
                             NoInputUpdate=self.initialTask.getTrustSitelists().get('trustlists'),
                             NoPileupUpdate=self.initialTask.getTrustSitelists().get('trustPUlists')
                            )

    def validate(self):
        """Check args and spec work with block splitting"""
        StartPolicyInterface.validateCommon(self)
        if not self.initialTask.inputDataset():
            raise WorkQueueWMSpecError(self.wmspec, 'No input dataset')

    def validBlocks(self, task, dbs):
        """Return blocks that pass the input data restriction"""
        datasetPath = task.getInputDatasetPath()
        Lexicon.dataset(datasetPath)  # check dataset name
        validBlocks = []

        blockWhiteList = task.inputBlockWhitelist()
        blockBlackList = task.inputBlockBlacklist()
        runWhiteList = task.inputRunWhitelist()
        runBlackList = task.inputRunBlacklist()
        lumiMask = task.getLumiMask()
        if lumiMask:
            maskedBlocks = self.getMaskedBlocks(task, dbs, datasetPath)

        for blockName in dbs.listFileBlocks(datasetPath):
            # check block restrictions
            if blockWhiteList and blockName not in blockWhiteList:
                continue
            if blockName in blockBlackList:
                continue

            blockSummary = dbs.getDBSSummaryInfo(block=blockName)
            if int(blockSummary.get('NumberOfFiles', 0)) == 0:
                logging.warning("Block %s being rejected for lack of valid files to process", blockName)
                self.badWork.append(blockName)
                continue

            if self.args['SliceType'] == 'NumberOfRuns':
                blockSummary['NumberOfRuns'] = dbs.listRuns(block=blockName)

            # check lumi restrictions
            if lumiMask:
                if blockName not in maskedBlocks:
                    logging.warning("Block %s doesn't pass the lumi mask constraints", blockName)
                    self.rejectedWork.append(blockName)
                    continue

                acceptedLumiCount = sum([len(maskedBlocks[blockName][lfn].getLumis()) for lfn in maskedBlocks[blockName]])
                ratioAccepted = 1. * acceptedLumiCount / float(blockSummary['NumberOfLumis'])
                maskedRuns = [maskedBlocks[blockName][lfn].getRuns() for lfn in maskedBlocks[blockName]]
                acceptedRuns = set(lumiMask.getRuns()).intersection(set().union(*maskedRuns))

                blockSummary['NumberOfFiles'] = len(maskedBlocks[blockName])
                blockSummary['NumberOfEvents'] = float(blockSummary['NumberOfEvents']) * ratioAccepted
                blockSummary[self.lumiType] = acceptedLumiCount
                blockSummary['NumberOfRuns'] = acceptedRuns
            # check run restrictions
            elif runWhiteList or runBlackList:
                runs = set(dbs.listRuns(block=blockName))
                # multi run blocks need special account, requires more DBS calls
                recalculateLumiCounts = True if len(runs) > 1 else False

                # apply blacklist and whitelist
                runs = runs.difference(runBlackList)
                if runWhiteList:
                    runs = runs.intersection(runWhiteList)
                # any runs left are ones we will run on, if none ignore block
                if not runs:
                    logging.warning("Block %s doesn't pass the runs constraints", blockName)
                    self.rejectedWork.append(blockName)
                    continue

                if recalculateLumiCounts:
                    # Recalculate the number of files, lumis and ~events accepted
                    acceptedLumiCount = 0
                    acceptedEventCount = 0
                    acceptedFileCount = 0
                    fileInfo = dbs.listFilesInBlock(fileBlockName=blockName)

                    for fileEntry in fileInfo:
                        acceptedFile = False
                        for lumiInfo in fileEntry['LumiList']:
                            if lumiInfo['RunNumber'] in runs:
                                acceptedFile = True
                                acceptedLumiCount += len(lumiInfo['LumiSectionNumber'])
                        if acceptedFile:
                            acceptedFileCount += 1
                            acceptedEventCount += fileEntry['NumberOfEvents']

                else:
                    acceptedLumiCount = blockSummary["NumberOfLumis"]
                    acceptedFileCount = blockSummary['NumberOfFiles']
                    acceptedEventCount = blockSummary['NumberOfEvents']

                blockSummary[self.lumiType] = acceptedLumiCount
                blockSummary['NumberOfFiles'] = acceptedFileCount
                blockSummary['NumberOfEvents'] = acceptedEventCount
                blockSummary['NumberOfRuns'] = runs

            validBlocks.append(blockSummary)

        # all needed blocks present at these sites
        if task.getTrustSitelists().get('trustlists'):
            siteWhitelist = task.siteWhitelist()
            siteBlacklist = task.siteBlacklist()
            self.sites = makeLocationsList(siteWhitelist, siteBlacklist)
            self.data[datasetPath] = self.sites
        else:
            # as the container is dealt as a whole, resolve the container location
            locations = set(self.blockLocationRucioPhedex(datasetPath))
            self.data[datasetPath] = list(set(self.cric.PNNstoPSNs(locations)))

        return validBlocks



#!/usr/bin/env python

"""
Version of dbsClient.dbsApi intended to be used with mock or unittest.mock
"""

from __future__ import (division, print_function)
from builtins import object
from future.utils import viewitems

import copy
import json
import os

from RestClient.ErrorHandling.RestClientExceptions import HTTPError
from WMCore.Services.DBS.DBSErrors import DBSReaderError
from WMCore.WMBase import getTestBase

from Utils.Utilities import encodeUnicodeToBytesConditional
from Utils.PythonVersion import PY2

# Read in the data just once so that we don't have to do it for every test (in __init__)

mockData = {}
globalFile = os.path.join(getTestBase(), '..', 'data', 'Mock', 'DBSMockData.json')
phys03File = os.path.join(getTestBase(), '..', 'data', 'Mock', 'DBSMockData03.json')

try:
    with open(globalFile, 'r') as mockFile:
        mockDataGlobal = json.load(mockFile)
except IOError:
    mockDataGlobal = {}
try:
    with open(phys03File, 'r') as mockFile:
        mockData03 = json.load(mockFile)
except IOError:
    mockData03 = {}

mockData['https://cmsweb-prod.cern.ch/dbs/prod/global/DBSReader'] = mockDataGlobal
mockData['https://cmsweb-prod.cern.ch/dbs/prod/phys03/DBSReader'] = mockData03

class MockDbsApi(object):
    def __init__(self, url):
        print("Using MockDBSApi")
        self.url = url.strip('/')

        # print("Initializing MockDBSApi")

    def serverinfo(self):
        return {'dbs_instance': 'MOCK', 'dbs_version': '3.3.144'}

    def listFileArray(self, **kwargs):
        """
        Handle the case when logical_file_name is called with a list (longer than one) of files
        since we don't want to store all permutations. Rebuild the list of dicts that DBS returns

        Args:
            **kwargs: any kwargs that dbs client accepts

        Returns:

        """
        self.item = 'listFileArray'

        if 'logical_file_name' in kwargs and len(kwargs['logical_file_name']) > 1:
            origArgs = copy.deepcopy(kwargs)
            returnDicts = []
            for lfn in kwargs['logical_file_name']:
                origArgs.update({'logical_file_name': [lfn]})
                returnDicts.extend(self.genericLookup(**origArgs))
            return returnDicts
        else:
            return self.genericLookup(**kwargs)

    def listFileLumiArray(self, **kwargs):
        """
        Handle the case when logical_file_name is called with a list (longer than one) of files
        since we don't want to store all permutations. Rebuild the list of dicts that DBS returns

        Args:
            **kwargs: any kwargs that dbs client accepts

        Returns:

        """
        self.item = 'listFileLumiArray'

        if 'logical_file_name' in kwargs and len(kwargs['logical_file_name']) > 1:
            origArgs = copy.deepcopy(kwargs)
            returnDicts = []
            # since we iterate over this, we better make sure it's a list to avoid
            # things like: ['U', 'N', 'K', 'N', 'O', 'W', 'N']
            if isinstance(kwargs['logical_file_name'], str):
                kwargs['logical_file_name'] = [kwargs['logical_file_name']]
            for lfn in kwargs['logical_file_name']:
                origArgs.update({'logical_file_name': [lfn]})
                returnDicts.extend(self.genericLookup(**origArgs))
            return returnDicts
        else:
            return self.genericLookup(**kwargs)

    def __getattr__(self, item):
        """
        __getattr__ gets called in case lookup of the actual method fails. We use this to return data based on
        a lookup table

        :param item: The method name the user is trying to call
        :return: The generic lookup function
        """
        self.item = item
        return self.genericLookup

    def genericLookup(self, *args, **kwargs):
        """
        This function returns the mocked DBS data

        :param args: positional arguments it was called with
        :param kwargs: named arguments it was called with
        :return: the dictionary that DBS would have returned
        """
        if self.url not in mockData:
            raise DBSReaderError("Mock DBS emulator knows nothing about instance %s" % self.url)

        if kwargs:
            for k in kwargs:
                if isinstance(kwargs[k], (list, tuple)):
                    kwargs[k] = [encodeUnicodeToBytesConditional(item, condition=PY2) for item in kwargs[k]]
                else:
                    kwargs[k] = encodeUnicodeToBytesConditional(kwargs[k], condition=PY2)
            signature = '%s:%s' % (self.item, sorted(viewitems(kwargs)))
        else:
            signature = self.item

        try:
            if mockData[self.url][signature] == 'Raises HTTPError':
                raise HTTPError('http:/dbs.mock.fail', 400, 'MockDBS is raising an exception in place of DBS', 'Dummy header', 'Dummy body')
            else:
                return mockData[self.url][signature]
        except KeyError:
            if kwargs.get('dataset', None) == '/HighPileUp/Run2011A-v1/RAW-BLAH':
                return []
            raise KeyError("DBS mock API could not return data for method %s, args=%s, and kwargs=%s (URL %s) (Signature: %s)" %
                           (self.item, args, kwargs, self.url, signature))



#! /usr/bin/env python

from builtins import zip, str, range
from future.utils import viewitems

import unittest

# import FWCore.ParameterSet.Config as cms
from WMCore.DataStructs.LumiList import LumiList


class LumiListTest(unittest.TestCase):
    """
    _LumiListTest_

    """

    def setUp(self):
        jsonFile = open('lumiTest.json', 'w')
        jsonFile.write('{"1": [[1, 33], [35, 35], [37, 47]], "2": [[49, 75], [77, 130], [133, 136]]}')
        jsonFile.close()

    def notestRead(self):
        """
        Test reading from JSON
        """
        exString = "1:1-1:33,1:35,1:37-1:47,2:49-2:75,2:77-2:130,2:133-2:136"
        exDict = {'1': [[1, 33], [35, 35], [37, 47]],
                  '2': [[49, 75], [77, 130], [133, 136]]}
        exVLBR = cms.VLuminosityBlockRange('1:1-1:33', '1:35', '1:37-1:47', '2:49-2:75', '2:77-2:130', '2:133-2:136')

        jsonList = LumiList(filename='lumiTest.json')
        lumiString = jsonList.getCMSSWString()
        lumiList = jsonList.getCompactList()
        lumiVLBR = jsonList.getVLuminosityBlockRange(True)

        self.assertTrue(lumiString == exString)
        self.assertTrue(lumiList == exDict)
        self.assertTrue(lumiVLBR == exVLBR)

    def testList(self):
        """
        Test constructing from list of pairs
        """

        listLs1 = list(range(1, 34)) + [35] + list(range(37, 48))
        listLs2 = list(range(49, 76)) + list(range(77, 131)) + list(range(133, 137))
        lumis = list(zip([1] * 100, listLs1)) + list(zip([2] * 100, listLs2))

        jsonLister = LumiList(filename='lumiTest.json')
        jsonString = jsonLister.getCMSSWString()
        jsonList = jsonLister.getCompactList()

        pairLister = LumiList(lumis=lumis)
        pairString = pairLister.getCMSSWString()
        pairList = pairLister.getCompactList()

        self.assertTrue(jsonString == pairString)
        self.assertTrue(jsonList == pairList)

    def testRuns(self):
        """
        Test constructing from run and list of lumis
        """
        runsAndLumis = {
            1: list(range(1, 34)) + [35] + list(range(37, 48)),
            2: list(range(49, 76)) + list(range(77, 131)) + list(range(133, 137))
        }
        runsAndLumis2 = {
            '1': list(range(1, 34)) + [35] + list(range(37, 48)),
            '2': list(range(49, 76)) + list(range(77, 131)) + list(range(133, 137))
        }
        blank = {
            '1': [],
            '2': []
        }

        jsonLister = LumiList(filename='lumiTest.json')
        jsonString = jsonLister.getCMSSWString()
        jsonList = jsonLister.getCompactList()

        runLister = LumiList(runsAndLumis=runsAndLumis)
        runString = runLister.getCMSSWString()
        runList = runLister.getCompactList()

        runLister2 = LumiList(runsAndLumis=runsAndLumis2)
        runList2 = runLister2.getCompactList()

        runLister3 = LumiList(runsAndLumis=blank)

        self.assertTrue(jsonString == runString)
        self.assertTrue(jsonList == runList)
        self.assertTrue(runList2 == runList)
        self.assertTrue(len(runLister3) == 0)

    def testFilter(self):
        """
        Test filtering of a list of lumis
        """
        runsAndLumis = {
            1: list(range(1, 34)) + [35] + list(range(37, 48)),
            2: list(range(49, 76)) + list(range(77, 131)) + list(range(133, 137))
        }

        completeList = list(zip([1] * 150, list(range(1, 150)))) + \
                       list(zip([2] * 150, list(range(1, 150)))) + \
                       list(zip([3] * 150, list(range(1, 150))))

        smallList = list(zip([1] * 50, list(range(1, 10)))) + list(zip([2] * 50, list(range(50, 70))))
        overlapList = list(zip([1] * 150, list(range(30, 40)))) + \
                      list(zip([2] * 150, list(range(60, 80))))
        overlapRes = list(zip([1] * 9, list(range(30, 34)))) + [(1, 35)] + \
                     list(zip([1] * 9, list(range(37, 40)))) + \
                     list(zip([2] * 30, list(range(60, 76)))) + \
                     list(zip([2] * 9, list(range(77, 80))))

        runLister = LumiList(runsAndLumis=runsAndLumis)

        # Test a list to be filtered which is a superset of constructed list
        filterComplete = runLister.filterLumis(completeList)
        # Test a list to be filtered which is a subset of constructed list
        filterSmall = runLister.filterLumis(smallList)
        # Test a list to be filtered which is neither
        filterOverlap = runLister.filterLumis(overlapList)

        self.assertTrue(filterComplete == runLister.getLumis())
        self.assertTrue(filterSmall == smallList)
        self.assertTrue(filterOverlap == overlapRes)

    def testDuplicates(self):
        """
        Test a list with lots of duplicates
        """
        result = list(zip([1] * 100, list(range(1, 34)) + list(range(37, 48))))
        lumis = list(zip([1] * 100, list(range(1, 34)) + list(range(37, 48)) + list(range(5, 25))))

        lister = LumiList(lumis=lumis)
        self.assertTrue(lister.getLumis() == result)

    def testNull(self):
        """
        Test a null list
        """

        runLister = LumiList(lumis=None)

        self.assertTrue(runLister.getCMSSWString() == '')
        self.assertTrue(runLister.getLumis() == [])
        self.assertTrue(runLister.getCompactList() == {})

    def testSubtract(self):
        """
        a-b for lots of cases
        """

        alumis = {'1': list(range(2, 20)) + list(range(31, 39)) + list(range(45, 49)),
                  '2': list(range(6, 20)) + list(range(30, 40)),
                  '3': list(range(10, 20)) + list(range(30, 40)) + list(range(50, 60)),
                  }
        blumis = {'1': list(range(1, 6)) + list(range(12, 13)) + list(range(16, 30)) + list(range(40, 50)) + list(
            range(33, 36)),
                  '2': list(range(10, 35)),
                  '3': list(range(10, 15)) + list(range(35, 40)) + list(range(45, 51)) + list(range(59, 70)),
                  }
        clumis = {'1': list(range(1, 6)) + list(range(12, 13)) + list(range(16, 30)) + list(range(40, 50)) + list(
            range(33, 36)),
                  '2': list(range(10, 35)),
                  }
        result = {'1': list(range(6, 12)) + list(range(13, 16)) + list(range(31, 33)) + list(range(36, 39)),
                  '2': list(range(6, 10)) + list(range(35, 40)),
                  '3': list(range(15, 20)) + list(range(30, 35)) + list(range(51, 59)),
                  }
        result2 = {'1': list(range(6, 12)) + list(range(13, 16)) + list(range(31, 33)) + list(range(36, 39)),
                   '2': list(range(6, 10)) + list(range(35, 40)),
                   '3': list(range(10, 20)) + list(range(30, 40)) + list(range(50, 60)),
                   }
        a = LumiList(runsAndLumis=alumis)
        b = LumiList(runsAndLumis=blumis)
        c = LumiList(runsAndLumis=clumis)
        r = LumiList(runsAndLumis=result)
        r2 = LumiList(runsAndLumis=result2)

        self.assertTrue((a - b).getCMSSWString() == r.getCMSSWString())
        self.assertTrue((a - b).getCMSSWString() != (b - a).getCMSSWString())
        # Test where c is missing runs from a
        self.assertTrue((a - c).getCMSSWString() == r2.getCMSSWString())
        self.assertTrue((a - c).getCMSSWString() != (c - a).getCMSSWString())
        # Test empty lists
        self.assertTrue(str(a - a) == '{}')
        self.assertTrue(len(a - a) == 0)

    def testOr(self):
        """
        a|b for lots of cases
        """

        alumis = {'1': list(range(2, 20)) + list(range(31, 39)) + list(range(45, 49)),
                  '2': list(range(6, 20)) + list(range(30, 40)),
                  '3': list(range(10, 20)) + list(range(30, 40)) + list(range(50, 60)),
                  }
        blumis = {'1': list(range(1, 6)) + list(range(12, 13)) + list(range(16, 30)) + list(range(40, 50)) + list(
            range(39, 80)),
                  '2': list(range(10, 35)),
                  '3': list(range(10, 15)) + list(range(35, 40)) + list(range(45, 51)) + list(range(59, 70)),
                  }
        clumis = {'1': list(range(1, 6)) + list(range(12, 13)) + list(range(16, 30)) + list(range(40, 50)) + list(
            range(39, 80)),
                  '2': list(range(10, 35)),
                  }
        result = {'1': list(range(2, 20)) + list(range(31, 39)) + list(range(45, 49)) + list(range(1, 6)) + list(
            range(12, 13)) + list(range(16, 30)) + list(range(40, 50)) + list(range(39, 80)),
                  '2': list(range(6, 20)) + list(range(30, 40)) + list(range(10, 35)),
                  '3': list(range(10, 20)) + list(range(30, 40)) + list(range(50, 60)) + list(range(10, 15)) + list(
                      range(35, 40)) + list(range(45, 51)) + list(range(59, 70)),
                  }
        a = LumiList(runsAndLumis=alumis)
        b = LumiList(runsAndLumis=blumis)
        c = LumiList(runsAndLumis=blumis)
        r = LumiList(runsAndLumis=result)
        self.assertTrue((a | b).getCMSSWString() == r.getCMSSWString())
        self.assertTrue((a | b).getCMSSWString() == (b | a).getCMSSWString())
        self.assertTrue((a | b).getCMSSWString() == (a + b).getCMSSWString())

        # Test list constuction (faster)

        multiple = [alumis, blumis, clumis]
        easy = LumiList(runsAndLumis=multiple)
        hard = a + b
        hard += c
        self.assertTrue(hard.getCMSSWString() == easy.getCMSSWString())

    def testAnd(self):
        """
        a&b for lots of cases
        """

        alumis = {'1': list(range(2, 20)) + list(range(31, 39)) + list(range(45, 49)),
                  '2': list(range(6, 20)) + list(range(30, 40)),
                  '3': list(range(10, 20)) + list(range(30, 40)) + list(range(50, 60)),
                  '4': list(range(1, 100)),
                  }
        blumis = {'1': list(range(1, 6)) + list(range(12, 13)) + list(range(16, 25)) + list(range(25, 40)) + list(
            range(40, 50)) + list(range(33, 36)),
                  '2': list(range(10, 35)),
                  '3': list(range(10, 15)) + list(range(35, 40)) + list(range(45, 51)) + list(range(59, 70)),
                  '5': list(range(1, 100)),
                  }
        result = {'1': list(range(2, 6)) + list(range(12, 13)) + list(range(16, 20)) + list(range(31, 39)) + list(
            range(45, 49)),
                  '2': list(range(10, 20)) + list(range(30, 35)),
                  '3': list(range(10, 15)) + list(range(35, 40)) + list(range(50, 51)) + list(range(59, 60)),
                  }
        a = LumiList(runsAndLumis=alumis)
        b = LumiList(runsAndLumis=blumis)
        r = LumiList(runsAndLumis=result)
        self.assertTrue((a & b).getCMSSWString() == r.getCMSSWString())
        self.assertTrue((a & b).getCMSSWString() == (b & a).getCMSSWString())
        self.assertTrue((a | b).getCMSSWString() != r.getCMSSWString())

    def testRemoveSelect(self):
        """
        a-b on runs for lots of cases
        """

        alumis = {'1': list(range(2, 20)) + list(range(31, 39)) + list(range(45, 49)),
                  '2': list(range(6, 20)) + list(range(30, 40)),
                  '3': list(range(10, 20)) + list(range(30, 40)) + list(range(50, 60)),
                  '4': list(range(10, 20)) + list(range(30, 80)),
                  }

        result = {'2': list(range(6, 20)) + list(range(30, 40)),
                  '4': list(range(10, 20)) + list(range(30, 80)),
                  }

        rem = LumiList(runsAndLumis=alumis)
        sel = LumiList(runsAndLumis=alumis)
        res = LumiList(runsAndLumis=result)

        rem.removeRuns([1, 3])
        sel.selectRuns([2, 4])

        self.assertTrue(rem.getCMSSWString() == res.getCMSSWString())
        self.assertTrue(sel.getCMSSWString() == res.getCMSSWString())
        self.assertTrue(sel.getCMSSWString() == rem.getCMSSWString())

    def testURL(self):
        URL = 'https://cms-service-dqm.web.cern.ch/cms-service-dqm/CAF/certification/Collisions12/8TeV/Reprocessing/Cert_190456-195530_8TeV_08Jun2012ReReco_Collisions12_JSON.txt'
        ll = LumiList(url=URL)
        self.assertTrue(len(ll) > 0)

    def testWrite(self):
        alumis = {'1': list(range(2, 20)) + list(range(31, 39)) + list(range(45, 49)),
                  '2': list(range(6, 20)) + list(range(30, 40)),
                  '3': list(range(10, 20)) + list(range(30, 40)) + list(range(50, 60)),
                  '4': list(range(1, 100)),
                  }
        a = LumiList(runsAndLumis=alumis)
        a.writeJSON('newFile.json')

    def testCompact(self):
        acl = {'1': [[1, 2], [3, 4], [8, 9]]}
        bcl = {'1': [[8, 9], [3, 4], [1, 2]]}
        ccl = {'1': [[1, 4], [8, 9]]}
        dcl = {'1': [[1, 4], [2, 3], [8, 9]]}

        self.assertEqual(LumiList(compactList=acl).getCMSSWString(), LumiList(compactList=bcl).getCMSSWString())
        self.assertEqual(LumiList(compactList=acl).getCMSSWString(), LumiList(compactList=ccl).getCMSSWString())
        self.assertEqual(LumiList(compactList=acl).getCMSSWString(), LumiList(compactList=dcl).getCMSSWString())

    def testWMagentFormat(self):
        """
        Test the constuction of the double-string-based version of lumi lists
        used in some places by WMAgent
        """

        clumis = {'1': list(range(2, 20)) + list(range(31, 39)) + list(range(45, 49)),
                  '2': list(range(6, 20)) + list(range(30, 40)),
                  '3': list(range(10, 20)) + list(range(30, 40)) + list(range(50, 60)),
                  '4': list(range(1, 100)),
                  }
        runs = [1, 2, 3, 4]
        lumis = ['2,19,31,38,45,48', '6,19,30,39', '10,19,30,39,50,59', '1,99']

        c = LumiList(runsAndLumis=clumis)
        w = LumiList(wmagentFormat=(runs, lumis))
        self.assertEqual(w.getCMSSWString(), c.getCMSSWString())

        clumis = {'12': [4, 5]}
        runs = [12]
        lumis = ['4,5']

        c = LumiList(runsAndLumis=clumis)
        w = LumiList(wmagentFormat=(runs, lumis))
        self.assertEqual(w.getCMSSWString(), c.getCMSSWString())

        aruns = [1]
        alumis = ['1,10,11,20']
        bruns = [1]
        blumis = ['1,20']
        self.assertEqual(LumiList(wmagentFormat=(aruns, alumis)).getCMSSWString(),
                         LumiList(wmagentFormat=(bruns, blumis)).getCMSSWString())

        with self.assertRaises(RuntimeError):
            w = LumiList(wmagentFormat=([1, 2, 3]))  # No lumis
        with self.assertRaises(RuntimeError):
            w = LumiList(wmagentFormat=([1], ['1,2,3']))  # Need twice as many lumis as runs

    def testLumisWithEvents(self):
        """Make sure that runs and lumis with event counts as used in CRAB3 works
        """
        clumis = {'1': list(range(2, 20)) + list(range(31, 39)) + list(range(45, 49)),
                  '2': list(range(6, 20)) + list(range(30, 40)),
                  '3': list(range(10, 20)) + list(range(30, 40)) + list(range(50, 60)),
                  '4': list(range(1, 100)),
                  }
        # create a lumilist like {'21' : {'2':None ...} ...}
        lumis = {}
        for run, ls in viewitems(clumis):
            newrun = str(int(run) + 20)
            lumis[newrun] = {str(l): None for l in ls}

        c1 = LumiList(runsAndLumis=clumis)
        w1 = LumiList(runsAndLumis=lumis)
        c1 += w1

        c2 = LumiList(runsAndLumis=clumis)
        w2 = LumiList(runsAndLumis=lumis)
        w2 += c2

        self.assertEqual(c1.getCMSSWString(), w2.getCMSSWString())


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_Timestamps_t_

Tests timestamps metrics
"""

import unittest

from WMCore.Configuration import Configuration
from Utils.Timestamps import addTimestampMetrics


class TimestampsTest(unittest.TestCase):
    """
    Timestamps class provides unit tests for Timestamps module
    """
    def testAddTimestamps(self):
        """
        test timestamps metrics in configuration
        """
        config = Configuration()
        wmJobStart = 1
        wmJobEnd = 2
        addTimestampMetrics(config, wmJobStart, wmJobEnd)
        self.assertEqual(config.WMTiming.WMJobStart, wmJobStart)
        self.assertEqual(config.WMTiming.WMJobEnd, wmJobEnd)
        self.assertEqual(config.WMTiming.WMTotalWallClockTime, wmJobEnd - wmJobStart)


if __name__ == "__main__":
    unittest.main()



from builtins import object
from WMCore.REST.Server import RESTFrontPage
from WMCore.REST.Server import MiniRESTApi
from WMCore.REST.Server import RESTApi
from WMCore.REST.Server import DBConnectionPool
from WMCore.REST.Server import DatabaseRESTApi
from WMCore.REST.Server import RESTEntity
import os, threading

srcfile = os.path.abspath(__file__).rsplit("/", 1)[-1].split(".")[0]
dbspec = {}

class FakeApp(object):
    appname = "app"

class FakeConf(object):
    db = srcfile + ".dbspec"

RESTFrontPage(None, None, "/", "/dev/null", {})
MiniRESTApi(FakeApp(), None, "/")
RESTApi(FakeApp(), None, "/")
DBConnectionPool("x", {})
if threading.current_thread().name == "MainThread":
    DatabaseRESTApi(FakeApp(), FakeConf(), "/")
RESTEntity(FakeApp(), None, None, "/")



#!/usr/bin/env python
"""
_LoadFromName_

MySQL implementation of Jobs.LoadFromName.
"""

__all__ = []

from WMCore.Database.DBFormatter import DBFormatter


class LoadFromName(DBFormatter):
    """
    _LoadFromName_

    Retrieve meta data for a job given it's name.  This includes the name,
    job group and last update time.
    """
    sql = """SELECT wmbs_job.id, jobgroup, wmbs_job.name AS name,
                    wmbs_job_state.name AS state, wmbs_job.state_time, retry_count,
                    couch_record, cache_dir, wmbs_location.site_name AS location,
                    outcome AS bool_outcome, fwjr_path AS fwjr_path
             FROM wmbs_job
               LEFT OUTER JOIN wmbs_location ON
                 wmbs_job.location = wmbs_location.id
               LEFT OUTER JOIN wmbs_job_state ON
                 wmbs_job.state = wmbs_job_state.id
             WHERE wmbs_job.name = :name"""

    def formatDict(self, result):
        """
        _formatDict_

        Cast the id, jobgroup and last_update columns to integers because
        formatDict() turns everything into strings.
        """
        formattedResult = DBFormatter.formatDict(self, result)[0]

        if formattedResult["bool_outcome"] == 0:
            formattedResult["outcome"] = "failure"
        else:
            formattedResult["outcome"] = "success"

        del formattedResult["bool_outcome"]
        return formattedResult

    def execute(self, name, conn=None, transaction=False):
        result = self.dbi.processData(self.sql, {"name": name}, conn=conn,
                                      transaction=transaction)
        return self.formatDict(result)



"""
__JobUpdater_t__

Test module for the JobUpdater component
Created on Apr 16, 2013

@author: dballest
"""

import logging
import os
import threading

from WMComponent.JobUpdater.JobUpdaterPoller import JobUpdaterPoller
from WMCore.DAOFactory import DAOFactory
from WMCore.WMBS.Fileset import Fileset
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.Workflow import Workflow
from WMCore.WMBase import getTestBase
from WMQuality.Emulators import EmulatorSetup
from WMQuality.TestInitCouchApp import TestInitCouchApp as TestInit
from WMQuality.Emulators.EmulatedUnitTestCase import EmulatedUnitTestCase


class JobUpdaterTest(EmulatedUnitTestCase):
    """
    _JobUpdaterTest_

    Test class for the JobUpdater
    """

    def setUp(self):
        """
        _setUp_

        Set up test environment
        """
        super(JobUpdaterTest, self).setUp()
        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules=["WMCore.WMBS", "WMCore.BossAir"],
                                useDefault=False)
        self.testInit.setupCouch('workqueue_t', 'WorkQueue')
        self.testInit.setupCouch('workqueue_inbox_t', 'WorkQueue')
        self.testDir = self.testInit.generateWorkDir(deleteOnDestruction=False)

        myThread = threading.currentThread()
        self.daoFactory = DAOFactory(package="WMCore.WMBS",
                                     logger=logging,
                                     dbinterface=myThread.dbi)
        self.listWorkflows = self.daoFactory(classname="Workflow.ListForSubmitter")
        self.configFile = EmulatorSetup.setupWMAgentConfig()

    def tearDown(self):
        """
        _tearDown_

        Tear down the databases
        """
        self.testInit.clearDatabase()
        self.testInit.tearDownCouch()
        self.testInit.delWorkDir()
        EmulatorSetup.deleteConfig(self.configFile)

    def getConfig(self):
        """
        _getConfig_

        Get a test configuration for
        the JobUpdater tests
        """
        config = self.testInit.getConfiguration()
        self.testInit.generateWorkDir(config)

        config.section_('Agent')
        config.Agent.agentName = 'testAgent'

        config.section_('CoreDatabase')
        config.CoreDatabase.connectUrl = os.environ['DATABASE']
        config.CoreDatabase.socket = os.getenv('DBSOCK')

        # JobTracker
        config.component_('JobUpdater')
        config.JobUpdater.reqMgrUrl = 'https://cmsweb-dev.cern.ch/reqmgr/reqMgr'

        # JobStateMachine
        config.section_('JobStateMachine')
        config.JobStateMachine.couchDBName = 'bogus'

        # BossAir
        config.section_('BossAir')
        config.BossAir.pluginNames = ['MockPlugin']
        config.BossAir.pluginDir = 'WMCore.BossAir.Plugins'
        config.BossAir.nCondorProcesses = 1
        config.BossAir.section_('MockPlugin')
        config.BossAir.MockPlugin.fakeReport = os.path.join(getTestBase(),
                                                            'WMComponent_t/JobAccountant_t/fwjrs',
                                                            'MergedSkimSuccess.pkl')

        # WorkQueue
        config.component_('WorkQueueManager')
        config.WorkQueueManager.couchurl = os.environ['COUCHURL']
        config.WorkQueueManager.dbname = 'workqueue_t'
        config.WorkQueueManager.inboxDatabase = 'workqueue_inbox_t'

        return config

    def stuffWMBS(self):
        """
        _stuffWMBS_

        Stuff WMBS with workflows
        """
        workflow = Workflow(spec='spec.xml', name='ReRecoTest_v0Emulator',
                            task='/ReRecoTest_v0Emulator/Test', priority=10)
        workflow.create()
        inputFileset = Fileset(name='TestFileset')
        inputFileset.create()
        subscription = Subscription(inputFileset, workflow)
        subscription.create()

    def test_BasicTest(self):
        """
        _BasicTest_

        Basic sanity check
        """
        self.stuffWMBS()
        poller = JobUpdaterPoller(self.getConfig())
        poller.reqmgr.getAssignment(self)
        result = self.listWorkflows.execute()
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0]['priority'], 10)
        poller.algorithm()
        result = self.listWorkflows.execute()
        self.assertEqual(result[0]['priority'], 100)



#!/usr/bin/env python
"""
_ReportEmu_t_

Tests for the JobReport emulator.
"""
from __future__ import print_function

import unittest

from WMCore.DataStructs.File import File
from WMCore.DataStructs.Run import Run
from WMCore.DataStructs.Job import Job

from WMCore.FwkJobReport.ReportEmu import ReportEmu
from WMQuality.Emulators.WMSpecGenerator.WMSpecGenerator import WMSpecGenerator
from WMQuality.Emulators.EmulatedUnitTestCase import EmulatedUnitTestCase

class ReportEmuTest(EmulatedUnitTestCase):
    """
    _ReportEmuTest_

    """
    def setUp(self):
        """
        _setUp_

        Setup some reasonable defaults for the ReReco workflow.
        """
        super(ReportEmuTest, self).setUp()

        self.unmergedLFNBase = "/store/backfill/2/unmerged"
        self.mergedLFNBase = "/store/backfill/2"
        self.processingVersion = "v1"
        self.cmsswVersion = "CMSSW_3_4_2_patch1"
        self.acquisitionEra = "WMAgentCommissioining10"
        self.primaryDataset = "MinimumBias"

        self.workload = WMSpecGenerator().createReRecoSpec("Tier1ReReco")
        print(self.workload.data)
        return

    def tearDown(self):
        """
        _tearDown_
        """
        super(ReportEmuTest, self).tearDown()

    def verifyOutputMetaData(self, outputFile, job):
        """
        _verifyOutputMetaData_

        Verify that metadata that in an emulated FWJR.  Most of the meta data
        should be the same as the input file.
        """
        goldenRuns = []

        for inputFile in job["input_files"]:
            for run in inputFile["runs"]:
                goldenRuns.append(Run(run.run, *run.lumis))

        assert len(outputFile["runs"]) == len(goldenRuns), \
                   "Error: Wrong number of runs in output file."

        for outputRun in outputFile["runs"]:
            for goldenRun in goldenRuns:
                if outputRun.run == goldenRun.run:
                    goldenRun.lumis.sort()
                    outputRun.lumis.sort()

                    if goldenRun.lumis == outputRun.lumis:
                        goldenRuns.remove(goldenRun)
                        break

        assert len(goldenRuns) == 0, \
               "Error: Run information wrong on output file."

        assert len(outputFile["locations"]) == 1,  \
               "Error: Wrong number of locations."

        assert list(outputFile["locations"])[0] == job["location"], \
               "Error: Output file at the wrong location."

        assert outputFile["merged"] == False, \
               "Error: Output should be unmerged."

        assert "adler32" in outputFile["checksums"], \
               "Error: Adler32 checksum missing."
        assert "cksum" in outputFile["checksums"], \
               "Error: CKSum checksum missing."

        return

    def testProcessing(self):
        """
        _testProcessing_

        Setup a processing workflow and job and verify that the FWJR produced
        by the emulator is reasonable.
        """
        rerecoTask = self.workload.getTask("DataProcessing")
        cmsRunStep = rerecoTask.getStep("cmsRun1")

        inputFile = File(lfn = "/path/to/test/lfn", size = 1048576, events = 1000, merged = True)
        inputFile.addRun(Run(1, *[1, 2, 3, 4, 5]))
        inputFile.addRun(Run(2, *[1, 2, 3, 4, 5, 6]))

        processingJob = Job(name = "ProcessingJob", files = [inputFile])
        processingJob["task"] = "/Tier1ReReco/ReReco"
        processingJob["mask"].setMaxAndSkipEvents(500, 0)
        processingJob["id"] = 1
        processingJob["location"] = "cmssrm.fnal.gov"

        emu = ReportEmu(WMStep = cmsRunStep.getTypeHelper(), Job = processingJob)
        report = emu()

        reportInputFiles = report.getInputFilesFromStep("cmsRun1")

        assert len(reportInputFiles) == 1, \
               "Error: Wrong number of input files for the job."
        assert reportInputFiles[0]["lfn"] == inputFile["lfn"], \
               "Error: Input LFNs do not match: %s" % reportInputFiles[0]["lfn"]
        assert reportInputFiles[0]["size"] == inputFile["size"], \
               "Error: Input file sizes do not match."
        assert reportInputFiles[0]["events"] == inputFile["events"], \
               "Error: Input file events do not match."

        goldenRuns = [Run(1, *[1, 2, 3, 4, 5]), Run(2, *[1, 2, 3, 4, 5, 6])]

        assert len(reportInputFiles[0]["runs"]) == len(goldenRuns), \
                   "Error: Wrong number of runs in input file."

        for inputRun in reportInputFiles[0]["runs"]:
            for goldenRun in goldenRuns:
                if inputRun.run == goldenRun.run:
                    goldenRun.lumis.sort()
                    inputRun.lumis.sort()

                    if goldenRun.lumis == inputRun.lumis:
                        goldenRuns.remove(goldenRun)
                        break

        assert len(goldenRuns) == 0, \
               "Error: Run information wrong on input file."

        recoOutputFiles = report.getFilesFromOutputModule("cmsRun1", "RECOoutput")
        alcaOutputFiles = report.getFilesFromOutputModule("cmsRun1", "ALCARECOoutput")

        assert len(recoOutputFiles) == 1, \
               "Error: There should only be one RECO output file."
        assert len(alcaOutputFiles) == 1, \
               "Error: There should only be one ALCA output file."

        assert recoOutputFiles[0]["module_label"] == "RECOoutput", \
               "Error: RECO file has wrong output module."
        assert alcaOutputFiles[0]["module_label"] == "ALCARECOoutput", \
               "Error: ALCA file has wrong output module."

        self.verifyOutputMetaData(recoOutputFiles[0], processingJob)
        self.verifyOutputMetaData(alcaOutputFiles[0], processingJob)

        dataTierMap = {"RECOoutput": "RECO", "ALCARECOoutput": "ALCARECO"}
        for outputFile in [recoOutputFiles[0], alcaOutputFiles[0]]:
            assert outputFile["dataset"]["applicationName"] == "cmsRun", \
                   "Error: Application name is incorrect."
            assert outputFile["dataset"]["primaryDataset"] == self.primaryDataset, \
                   "Error: Primary dataset is incorrect."
            assert outputFile["dataset"]["dataTier"] == dataTierMap[outputFile["module_label"]], \
                   "Error: Data tier is incorrect."

        return

    def testMerge(self):
        """
        _testMerge_

        Setup a merge workflow and job and verify that the FWJR produced by the
        emulator is reasonable.
        """
        #emu = ReportEmu(WMStep = self.cmssw, Job = self.job)
        #report = emu()
        return

if __name__ == "__main__":
    unittest.main()



"""
Created on Aug 13, 2014
@author: sryu
"""

from __future__ import (division, print_function)

from WMCore.REST.CherryPyPeriodicTask import CherryPyPeriodicTask

class LogDBTasks(CherryPyPeriodicTask):

    def __init__(self, rest, config):
        super(LogDBTasks, self).__init__(config)

    def setConcurrentTasks(self, config):
        """
        Set logDBCleanUp method to be executed in a polling cycle
        """
        self.concurrentTasks = [{'func': self.logDBCleanUp, 'duration': config.logDBCleanDuration}]

    def logDBCleanUp(self, config):
        """
        gather active data statistics
        """
        self.logger.info("Cleaning documents from LogDB WMStats")
        docs = self.logDB.cleanup(config.keepDocsAge)
        self.logger.info("Deleted %d old documents", len(docs))

        return



from WMCore.WebTools.RESTModel import RESTModel
from WMQuality.WebTools.RESTBaseUnitTest import RESTBaseUnitTest
from WMQuality.WebTools.RESTServerSetup import DefaultConfig
import WMCore

import logging
import unittest
import threading
import cherrypy
import os
import uuid
import tempfile
from cgi import FieldStorage

def noBodyProcess():
    """Sets cherrypy.request.process_request_body = False, giving
    us direct control of the file upload destination. By default
    cherrypy loads it to memory, we are directing it to disk."""
    cherrypy.request.process_request_body = False
cherrypy.tools.noBodyProcess = cherrypy.Tool('before_request_body', noBodyProcess)

class PyCurlRESTModel(RESTModel):
    """
    Check if the file upload work with the rest model
    """
    def __init__(self, config={}):
        RESTModel.__init__(self, config)

        self._addMethod('POST', 'file', self.uploadFile,
                        args=['file1'])

        cherrypy.engine.subscribe('start_thread', self.initThread)

    def initThread(self, thread_index):
        """
        The ReqMgr expects the DBI to be contained in the Thread
        """
        myThread = threading.currentThread()
        #myThread = cherrypy.thread_data
        # Get it from the DBFormatter superclass
        myThread.dbi = self.dbi

    @cherrypy.expose
    @cherrypy.tools.noBodyProcess()
    def uploadFile(self, file1):
        """
        Saves the file passed by the client in the current directory
        """
        try:
            with open('UploadedFile.txt', 'wb') as f:
                f.write( file1.file.read() )
        except Exception as e:
            logging.exception(e)

        return {'result':'Success'}



#!/usr/bin/env python
"""
_MinFileBased_

File based splitting algorithm that will chop a fileset into
a set of jobs based on file boundaries.  Requires the minimum number
of files to create a new job.  If you do not have the necessary
number of files, it will not create any jobs UNLESS the
fileset is closed.
"""

from WMCore.JobSplitting.JobFactory import JobFactory
from WMCore.WMBS.File               import File

class MinFileBased(JobFactory):
    def algorithm(self, *args, **kwargs):
        """
        _algorithm_

        Split up all the available files such that each job will process a
        maximum of 'files_per_job'.  If the 'files_per_job' parameters is not
        passed in jobs will process a maximum of 10 files.
        """

        filesPerJob   = int(kwargs.get("files_per_job", 10))
        jobsPerGroup  = int(kwargs.get("jobs_per_group", 0))
        runBoundaries = kwargs.get("respect_run_boundaries", False)
        getParents    = kwargs.get("include_parents", False)
        filesInJob    = 0
        listOfFiles   = []

        # Grab the fileset
        fileset = self.subscription.getFileset()
        try:
            fileset.load()
        except AttributeError as ae:
            pass

        #Get a dictionary of sites, files
        locationDict = self.sortByLocation()

        for location in locationDict:
            #Now we have all the files in a certain location
            fileList    = locationDict[location]
            filesInJob  = 0
            jobsInGroup = 0
            fileCounter = 0
            if len(fileList) == 0:
                #No files for this location
                #This isn't supposed to happen, but better safe then sorry
                continue
            if len(fileList) < filesPerJob and fileset.open:
                continue
            self.newGroup()
            jobRun = None
            for f in fileList:
                if getParents:
                    parentLFNs = self.findParent(lfn = f['lfn'])
                    for lfn in parentLFNs:
                        parent = File(lfn = lfn)
                        f['parents'].add(parent)
                fileRun = f.get('minrun', None)
                if filesInJob == 0 or filesInJob == filesPerJob or (runBoundaries and fileRun != jobRun):
                    if jobsPerGroup:
                        if jobsInGroup > jobsPerGroup:
                            self.newGroup()
                            jobsInGroup = 0

                    filesInJob   = 0
                    # We've finished a job.  Should we create another one?
                    if len(fileList) - fileCounter < filesPerJob and fileset.open:
                        # If we don't have the full job's worth of files
                        # and the fileset is still open
                        # then we shouldn't create a new job
                        continue
                    self.newJob(name = self.getJobName())

                    jobsInGroup += 1
                    jobRun       = fileRun

                filesInJob += 1
                self.currentJob.addFile(f)
                fileCounter += 1

                listOfFiles.append(f)

        return



#!/usr/bin/env python
"""
Unittest for CMSSW.py
"""

import unittest
import tempfile
import shutil
import os
import WMCore.WMSpec.Steps.StepFactory as StepFactory
import WMCore.WMSpec.WMStep as WMStep
import WMCore.WMSpec.Steps as TemplateNS


class CMSSWBuildTest(unittest.TestCase):
    ''' unittests for the CMSSW builder'''
    def setUp(self):
        '''create the builder object and the temporary directory'''
        self.tempDir = tempfile.mkdtemp()

        try:
            self.testBuilder = StepFactory.getStepBuilder("CMSSW")
        except Exception as ex:
            msg = "Failed to instantiate Builder:\n"
            msg += str(ex)
            self.fail(msg)

    def tearDown(self):
        '''remove the temp directory we created'''
        shutil.rmtree( self.tempDir )

    def testBuild(self):
        ''' build a directory and verify it exists'''
        mytemplate = StepFactory.getStepTemplate("CMSSW")
        mystep = WMStep.makeWMStep("DummyStagingStep")
        mytemplate(mystep.data)
        self.testBuilder(mystep.data, "testTask", self.tempDir)
        self.assertTrue(os.path.exists(self.tempDir))
        self.assertTrue(os.path.exists("%s/DummyStagingStep/__init__.py"
                                       % self.tempDir))

    def testCustomBuild(self):
        ''' add in a custom directory and verify it gets created'''
        mytemplate = StepFactory.getStepTemplate("CMSSW")
        mystep = WMStep.makeWMStep("DummyStagingStep")
        mytemplate(mystep.data)
        helper = TemplateNS.Template.CoreHelper(mystep.data)
        helper.addDirectory( 'testdirectory1' )
        helper.addDirectory( 'testdirectory2/testsubdir' )
        self.testBuilder(mystep.data, "testTask", self.tempDir)
        self.assertTrue(os.path.exists(self.tempDir))
        self.assertTrue(os.path.exists("%s/DummyStagingStep/__init__.py"
                                       % self.tempDir))
        self.assertTrue(os.path.exists("%s/DummyStagingStep/testdirectory1"
                                       % self.tempDir))
        self.assertTrue(os.path.exists("%s/%s/testdirectory2/testsubdir"
                                       % (self.tempDir, 'DummyStagingStep')))





if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()
